/*+
 * Name:	classes.c.c
 *
 * Purpose:	translates treelist of inheritance hierarchy into inheritance links 
 *
 * Author:	Thomas R. Dean
 *
 * Date:	April 18, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

void do_inherit(gd_graph *g,int i);

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  node_ptr *rover;
  str_ptr void_ptr;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  for(i = 0; i < g.curNodePos; i++){
    if (g.nodeBuffer[i].common.type == dC_AGGR_TYPE &&
	g.nodeBuffer[i].dc_aggr_type.inheritance != -1){
      do_inherit(&g,i);
    }
  }

  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }
  return 0;
}

void do_inherit(gd_graph *g,int i){
  /* inheritance points to a tree list that 
   * was generated from a varaible length tree_vec
   * by the dumper. Inheritance attributes
   * are in the tree vec flags, with the unsigned
   * flag indicating virtual inheritance */

  node_ptr rover;

  rover = g->nodeBuffer[i].dc_aggr_type.inheritance;
  while(rover != -1){
    /* fields to save */
    node_ptr dst = g->nodeBuffer[rover].tree_list.value;
    node_ptr next = g->nodeBuffer[rover].tree_list.chan;

    g->nodeBuffer[rover].common.type = dC_INHERITANCE;
    g->nodeBuffer[rover].common.other_node = -1;
    g->nodeBuffer[rover].dc_arc.ordinal = 0;
    g->nodeBuffer[rover].dc_arc.src = i;
    g->nodeBuffer[rover].dc_arc.dest = dst;
    g->nodeBuffer[rover].dc_arc.next = next;
    g->nodeBuffer[rover].dc_inheritance.visb=visb_unk;
    if (g->nodeBuffer[rover].common.public_flag){
      g->nodeBuffer[rover].dc_inheritance.visb=visb_pub;
    } else if (g->nodeBuffer[rover].common.protected_flag){
      g->nodeBuffer[rover].dc_inheritance.visb=visb_pro;
    } else {
      g->nodeBuffer[rover].dc_inheritance.visb=visb_pri;
    }
    g->nodeBuffer[rover].dc_inheritance.virt = 
      g->nodeBuffer[rover].common.unsigned_flag;
    rover = next;
  }
}
