// Make Actual Parameter Lists

// In gcc, the first op of a call expr is a function reference; and the second is a 
// tree list of actual parameters.  In Datrix, the latter should be an Actual Fct Param List
// with ArcSons to the actuals.  Here, we do it.  The node for the Actual Fct Param List
// is allocated.  The tree list nodes are reused for ArcSons

/* AJM 2001 Mar 19 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_actuals (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_actuals (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

void do_actuals (gd_graph *g)
{
    int i;
    node_ptr ptr;			// Ptr to node which is to be converted to "operator".
    gd_graph_node *expr_node_p;		// Pointer to node which is operator

    node_ptr actuals_ptr;		// Ptr to new 
    gd_graph_node new_actuals;		// New Datrix node for the actual fct param list.

    gd_graph_node new_edge;		// New Datrix edge for various purposes.
    node_ptr edge_ptr;			// ptr variable which pts to tree-list/arcson node

    int ord;				// ordinal of an actual parameter in the list

    // Do all the calls.
    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
    {
	expr_node_p = &(g-> nodeBuffer [ptr]);

	// Is this a call operation?
	if (
		expr_node_p-> common. type == dC_OPERATOR
	    &&	expr_node_p-> dc_operator. op == dc_fct_call
	    )
	{
	    // Yes.   The op1 attribute of this node is a tree list of the actuals.
	    // This will be converted into a linked collection of arcson edges.
	    // They are sons of a new node, which will be the actual fct param list.

	    // Walk the dC_ARC_OPD list (ops) to get to the second operand.
	    node_ptr opd0 = expr_node_p-> dc_operator. ops;
	    node_ptr opd1 = g-> nodeBuffer [opd0]. dc_arc. next;

	    // Create and save the new actual fct param list node.
	    new_actuals. common. type = dC_ACTUAL_FCT_PARAM_LIST;
	    new_actuals. common. other_node = -1;
	    new_actuals. dc_expression. refers = -1;
	    new_actuals. dc_actual_fct_param_list. actuals = g-> nodeBuffer [opd1]. dc_arc. dest;
	    actuals_ptr = newNode (g);
	    g-> nodeBuffer [actuals_ptr] = new_actuals;

	    // Walk the tree list converting each one to a dC_ARC_SON.
	    new_edge. common. type = dC_ARC_SON;
	    new_edge. common. other_node = -1;
	    ord = 0;

	    // Start at the beginning of the tree list.
	    edge_ptr = g-> nodeBuffer [opd1]. dc_arc. dest;


	    // Continue until the tree list is exhausted.
	    while (edge_ptr != -1)
	    {
		// Set arc data for the new edge.
		new_edge. dc_arc. ordinal = ++ord;
		new_edge. dc_arc. src = actuals_ptr;
		new_edge. dc_arc. dest = g-> nodeBuffer [edge_ptr]. tree_list. value;
		new_edge. dc_arc. next = g-> nodeBuffer [edge_ptr]. tree_list. chan;

		// Convert the node.
		g-> nodeBuffer [edge_ptr] = new_edge;

		// Step to next acutal (which is still a tree list node).
		edge_ptr = new_edge. dc_arc. next;
	    }

	    // point op1 at actuals (T.D.)
	    g-> nodeBuffer[opd1].dc_arc.dest = actuals_ptr;
	}
    }
}
