/* this code is almost identical to gd_reduce, and is
 * in fact derived from it. We have removed any special
 * consideration of the spine, since it is run after
 * namespace.  This has been done by removing the 
 * code that marks the spine and examines it afterwards.
 * Also done by queuing the first node
 * (as given by the graph header), and setting mark_flag2 to 0 for
 * all nodes */

#include <stdio.h>

#include <gd_graph.h>

void queue_add(node_ptr nodenum, int byChan);
void queue_first(node_ptr * nodenum, int * byChan);
int queue_empty(void);


void process_node(gd_graph g, node_ptr nodenum, int byChan);

char * progname;
str_ptr internal;
node_ptr last_node;

void usage(){
  fprintf(stderr,"usage: %s ingdfile outgdfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * infilename, * outfilename;
  int nodenum,byChan,i;
  node_ptr * node_rover;

  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  infilename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(infilename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  last_node = g.curNodePos;
  for(i = 0; i < g.curNodePos; i++) {
    g.nodeBuffer[i].common.mark_flag1 = 0;
    g.nodeBuffer[i].common.mark_flag2 = 0;
  }

  queue_add(g.firstNode,0);

  while(!queue_empty()){
    queue_first(&nodenum,&byChan);
    process_node(g,nodenum,byChan);
  }

  /* remove the unused nodes from the graph */
  for(i = 0; i < g.curNodePos; i++) {
    if (g.nodeBuffer[i].common.mark_flag1 == 0){
      g.nodeBuffer[i].common.type=NODE_DELTED;
    }
    /* clear mark bits to make binary comparisons easier */
    g.nodeBuffer[i].common.mark_flag1 = 0;
    g.nodeBuffer[i].common.mark_flag2 = 0;
  }

  /* write the file */
  if (gd_graph_write_binary_file(outfilename,g)==0){
    fprintf(stderr,"problem writing binary file");
    exit (1);
  }

  return 0;
}

struct t_queue_node{
  node_ptr nodenum;
  int byChan;
  struct t_queue_node * next;
};
typedef struct t_queue_node t_queue_node;

t_queue_node * head = NULL;
t_queue_node * tail = NULL;

void queue_add(node_ptr nodenum, int byChan){
  t_queue_node * tmp;

  if (nodenum < -1 || nodenum >= last_node){
    printf("wierd edge\n");
    return;
  };

  if (nodenum == -1) return;

  tmp = (t_queue_node *)malloc(sizeof(t_queue_node));
  tmp -> nodenum = nodenum;
  tmp -> byChan = byChan;
  tmp -> next = NULL;
  if (head == NULL) {
    head = tmp;
    tail = tmp;
  } else
  {
    tail-> next = tmp;
    tail = tmp;
  }
}

void queue_first(node_ptr * nodenum, int * byChan) {
  t_queue_node * tmp;
  *nodenum = head -> nodenum;
  *byChan = head -> byChan;
  tmp = head;
  head = head -> next;
  free(tmp);
}

int queue_empty(void){
  return head == NULL;
}

/* the walker */

void do_expr(t_gcc_expr *expr);
void do_common(gd_graph_common *common);
int do_decl(t_gcc_decl *decl, int byChan);
void do_edge(node_ptr nextNode);
void do_type(t_gcc_type *type);
void do_cst(t_gcc_const *cst);

void datrix_asg_nd(t_dc_asg_nd *nd);
void datrix_identifier(t_dc_identifier *ident);
void datrix_loop(t_dc_loop *loop);
void datrix_ctrl_stmt(t_dc_ctrl_stmt *cntrl);
void datrix_comment(t_dc_comment *comment);
void datrix_jump(t_dc_jump *jump);
void datrix_expression(t_dc_expression *expr);
void datrix_operator(t_dc_operator *operator);
void datrix_type(t_dc_type *type);
void datrix_object(t_dc_object *object);
void datrix_label(t_dc_label *label);
void datrix_scope(t_dc_scope *scope);

void datrix_arc(t_dc_arc  *dc_arc);
void datrix_arc_reference(t_dc_arc_reference  *dc_arc_reference);
void datrix_arc_syntax(t_dc_arc_syntax  *dc_arc_syntax);

void process_node(gd_graph g, node_ptr nodenum, int byChan)
{
  /* printf("node: %4d, chan %d ",nodenum,byChan); */
  /* only process each node once */
  if (g.nodeBuffer[nodenum].common.mark_flag1){
    /* printf("<\n"); */
    return;
  }
  /* printf("*\n"); */

  switch (g.nodeBuffer[nodenum].common.type){
    case gNAMESPACE_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].namespace_decl.dcls);
	do_edge(g.nodeBuffer[nodenum].namespace_decl.alis);
      }
      break;
    case gFUNCTION_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].function_decl.args);
	do_edge(g.nodeBuffer[nodenum].function_decl.body);
      }
      break;
    case gVAR_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].var_decl.init);
	do_edge(g.nodeBuffer[nodenum].var_decl.size);
      }
      break;
    case gPARM_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].parm_decl.argt);
	do_edge(g.nodeBuffer[nodenum].parm_decl.size);
      }
      break;
    case gTYPE_DECL:
      do_decl(&g.nodeBuffer[nodenum].decl,byChan);
      break;
    case gLABEL_DECL:
      do_decl(&g.nodeBuffer[nodenum].decl,byChan);
      break;
    case gCONST_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].const_decl.cnst);
      }
      break;
    case gRESULT_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].result_decl.init);
	do_edge(g.nodeBuffer[nodenum].result_decl.size);
      }
      break;
    case gFIELD_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].field_decl.init);
	do_edge(g.nodeBuffer[nodenum].field_decl.size);
	do_edge(g.nodeBuffer[nodenum].field_decl.bpos);
      }
      break;
    case gTEMPLATE_DECL:
      if (do_decl(&g.nodeBuffer[nodenum].decl,byChan)){
	do_edge(g.nodeBuffer[nodenum].template_decl.rslt);
	do_edge(g.nodeBuffer[nodenum].template_decl.inst);
	do_edge(g.nodeBuffer[nodenum].template_decl.spcs);
	do_edge(g.nodeBuffer[nodenum].template_decl.parms);
      }
      break;
    case gUSING_DECL:
      do_decl(&g.nodeBuffer[nodenum].decl,byChan);
      break;
    case gFUNCTION_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].function_type.retn);
      do_edge(g.nodeBuffer[nodenum].function_type.prms);
      break;
    case gINTEGER_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].integer_type.min);
      do_edge(g.nodeBuffer[nodenum].integer_type.max);
      break;
    case gARRAY_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].array_type.elts);
      do_edge(g.nodeBuffer[nodenum].array_type.domn);
      break;
    case gVOID_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gREAL_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gCOMPLEX_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gVECTOR_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gENUMERAL_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].enumeral_type.min);
      do_edge(g.nodeBuffer[nodenum].enumeral_type.max);
      do_edge(g.nodeBuffer[nodenum].enumeral_type.csts);
      break;
    case gBOOLEAN_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gCHAR_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gPOINTER_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].pointer_type.ptd);
      do_edge(g.nodeBuffer[nodenum].pointer_type.cls);
      break;
    case gOFFSET_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gREFERENCE_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].reference_type.refd);
      break;
    case gMETHOD_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].method_type.clas);
      do_edge(g.nodeBuffer[nodenum].method_type.retn);
      do_edge(g.nodeBuffer[nodenum].method_type.prms);
      break;
    case gFILE_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gSET_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gRECORD_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].record_type.flds);
      do_edge(g.nodeBuffer[nodenum].record_type.fncs);
      do_edge(g.nodeBuffer[nodenum].record_type.ptd);
      do_edge(g.nodeBuffer[nodenum].record_type.cls);
      do_edge(g.nodeBuffer[nodenum].record_type.vfld);
      do_edge(g.nodeBuffer[nodenum].record_type.binf);
      break;
    case gUNION_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      do_edge(g.nodeBuffer[nodenum].record_type.flds);
      do_edge(g.nodeBuffer[nodenum].record_type.fncs);
      do_edge(g.nodeBuffer[nodenum].record_type.ptd);
      do_edge(g.nodeBuffer[nodenum].record_type.cls);
      do_edge(g.nodeBuffer[nodenum].record_type.vfld);
      do_edge(g.nodeBuffer[nodenum].record_type.binf);
      break;
    case gQUAL_UNION_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gLANG_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gTEMPLATE_TYPE_PARM:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gTEMPLATE_PARM_INDEX:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gTEMPLATE_TEMPLATE_PARM:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gBOUND_TEMPLATE_TEMPLATE_PARM:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gTYPENAME_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gTYPEOF_TYPE:
      do_type(&g.nodeBuffer[nodenum].type);
      break;
    case gBLOCK:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gCOMPOUND_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].compound_stmt.body);
      do_edge(g.nodeBuffer[nodenum].compound_stmt.next);
      break;
    case gEXPR_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].expr_stmt.texpr);
      do_edge(g.nodeBuffer[nodenum].expr_stmt.next);
      break;
    case gDECL_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].decl_stmt.decl);
      do_edge(g.nodeBuffer[nodenum].decl_stmt.next);
      break;
    case gSCOPE_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].scope_stmt.next);
      break;
    case gRETURN_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].return_stmt.ret_expr);
      do_edge(g.nodeBuffer[nodenum].return_stmt.next);
      break;
    case gSTMT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].stmt_expr.stmt);
      break;
    case gBREAK_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].break_stmt.next);
      break;
    case gCONTINUE_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].continue_stmt.next);
      break;
    case gIF_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].if_stmt.cond);
      do_edge(g.nodeBuffer[nodenum].if_stmt.thenc);
      do_edge(g.nodeBuffer[nodenum].if_stmt.elsec);
      do_edge(g.nodeBuffer[nodenum].if_stmt.next);
      break;
    case gSWITCH_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].switch_stmt.cond);
      do_edge(g.nodeBuffer[nodenum].switch_stmt.body);
      do_edge(g.nodeBuffer[nodenum].switch_stmt.next);
      break;
    case gLABEL_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].label_stmt.label);
      do_edge(g.nodeBuffer[nodenum].label_stmt.next);
      break;
    case gTRY_BLOCK:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].try_block.body);
      do_edge(g.nodeBuffer[nodenum].try_block.hdlr);
      do_edge(g.nodeBuffer[nodenum].try_block.next);
      break;
    case gHANDLER:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].handler.body);
      do_edge(g.nodeBuffer[nodenum].handler.next);
      break;
    case gSTART_CATCH_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].start_catch_stmt.catch_type);
      do_edge(g.nodeBuffer[nodenum].start_catch_stmt.next);
      break;
    case gCLEANUP_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].cleanup_stmt.decl);
      do_edge(g.nodeBuffer[nodenum].cleanup_stmt.clean_expr);
      do_edge(g.nodeBuffer[nodenum].cleanup_stmt.next);
      break;
    case gCASE_LABEL:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].case_label.low);
      do_edge(g.nodeBuffer[nodenum].case_label.high);
      do_edge(g.nodeBuffer[nodenum].case_label.next);
      break;
    case gGOTO_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].goto_stmt.dest);
      do_edge(g.nodeBuffer[nodenum].goto_stmt.next);
      break;
    case gFOR_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].for_stmt.init);
      do_edge(g.nodeBuffer[nodenum].for_stmt.cond);
      do_edge(g.nodeBuffer[nodenum].for_stmt.inc_expr);
      do_edge(g.nodeBuffer[nodenum].for_stmt.body);
      do_edge(g.nodeBuffer[nodenum].for_stmt.next);
      break;
    case gWHILE_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].while_stmt.cond);
      do_edge(g.nodeBuffer[nodenum].while_stmt.body);
      do_edge(g.nodeBuffer[nodenum].while_stmt.next);
      break;
    case gDO_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].do_stmt.cond);
      do_edge(g.nodeBuffer[nodenum].do_stmt.body);
      do_edge(g.nodeBuffer[nodenum].do_stmt.next);
      break;
    case gASM_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].asm_stmt.strg);
      do_edge(g.nodeBuffer[nodenum].asm_stmt.outs);
      do_edge(g.nodeBuffer[nodenum].asm_stmt.ins);
      do_edge(g.nodeBuffer[nodenum].asm_stmt.clbr);
      do_edge(g.nodeBuffer[nodenum].asm_stmt.next);
      break;
    case gCOMPONENT_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].component_ref.op0);
      break;
    case gBIT_FIELD_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gINDIRECT_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].indirect_ref.op0);
      break;
    case gBUFFER_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gARRAY_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].array_ref.op0);
      do_edge(g.nodeBuffer[nodenum].array_ref.op1);
      break;
    case gOFFSET_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].offset_ref.op0);
      do_edge(g.nodeBuffer[nodenum].offset_ref.op1);
      break;
    case gSCOPE_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].scope_ref.op0);
      do_edge(g.nodeBuffer[nodenum].scope_ref.op1);
      break;
    case gMEMBER_REF:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].member_ref.op0);
      do_edge(g.nodeBuffer[nodenum].member_ref.op1);
      break;
    case gCONSTRUCTOR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].constructor.elts);
      break;
    case gCOMPOUND_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].compound_expr.op0);
      do_edge(g.nodeBuffer[nodenum].compound_expr.op1);
      break;
    case gMODIFY_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].modify_expr.op0);
      do_edge(g.nodeBuffer[nodenum].modify_expr.op1);
      break;
    case gINIT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].init_expr.op0);
      do_edge(g.nodeBuffer[nodenum].init_expr.op1);
      break;
    case gTHROW_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].throw_expr.op0);
      break;
    case gTARGET_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].target_expr.decl);
      do_edge(g.nodeBuffer[nodenum].target_expr.init);
      do_edge(g.nodeBuffer[nodenum].target_expr.clnp);
      break;
    case gCOND_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].cond_expr.op0);
      do_edge(g.nodeBuffer[nodenum].cond_expr.op1);
      do_edge(g.nodeBuffer[nodenum].cond_expr.op2);
      break;
    case gBIND_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bind_expr.vars);
      do_edge(g.nodeBuffer[nodenum].bind_expr.body);
      break;
    case gCALL_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].call_expr.fn);
      do_edge(g.nodeBuffer[nodenum].call_expr.args);
      break;
    case gMETHOD_CALL_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gWITH_CLEANUP_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gCLEANUP_POINT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].cleanup_point_expr.op0);
      break;
    case gPLACEHOLDER_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gWITH_RECORD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gPLUS_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].plus_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].plus_expr.expr2.op1);
      break;
    case gMINUS_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].minus_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].minus_expr.expr2.op1);
      break;
    case gMULT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].mult_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].mult_expr.expr2.op1);
      break;
    case gTRUNC_DIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].trunc_div_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].trunc_div_expr.expr2.op1);
      break;
    case gCEIL_DIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ceil_div_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].ceil_div_expr.expr2.op1);
      break;
    case gFLOOR_DIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].floor_div_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].floor_div_expr.expr2.op1);
      break;
    case gROUND_DIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].round_div_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].round_div_expr.expr2.op1);
      break;
    case gTRUNC_MOD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op1);
      break;
    case gCEIL_MOD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op1);
      break;
    case gFLOOR_MOD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].floor_mod_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].floor_mod_expr.expr2.op1);
      break;
    case gROUND_MOD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].round_mod_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].round_mod_expr.expr2.op1);
      break;
    case gRDIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].rdiv_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].rdiv_expr.expr2.op1);
      break;
    case gEXACT_DIV_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].exact_div_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].exact_div_expr.expr2.op1);
      break;
    case gFIX_TRUNC_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].fix_trunc_expr.expr1.op0);
      break;
    case gFIX_CEIL_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].fix_ceil_expr.expr1.op0);
      break;
    case gFIX_FLOOR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].fix_floor_expr.expr1.op0);
      break;
    case gFIX_ROUND_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].fix_round_expr.expr1.op0);
      break;
    case gFLOAT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].float_expr.expr1.op0);
      break;
    case gEXPON_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].expon_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].expon_expr.expr2.op1);
      break;
    case gNEGATE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].negate_expr.expr1.op0);
      break;
    case gMIN_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].min_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].min_expr.expr2.op1);
      break;
    case gMAX_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].max_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].max_expr.expr2.op1);
      break;
    case gABS_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].abs_expr.expr1.op0);
      break;
    case gFFS_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ffs_expr.expr1.op0);
      break;
    case gLSHIFT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].lshift_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].lshift_expr.expr2.op1);
      break;
    case gRSHIFT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].rshift_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].rshift_expr.expr2.op1);
      break;
    case gLROTATE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].lrotate_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].lrotate_expr.expr2.op1);
      break;
    case gRROTATE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].rrotate_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].rrotate_expr.expr2.op1);
      break;
    case gBIT_IOR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bit_ior_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].bit_ior_expr.expr2.op1);
      break;
    case gBIT_XOR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bit_xor_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].bit_xor_expr.expr2.op1);
      break;
    case gBIT_AND_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bit_and_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].bit_and_expr.expr2.op1);
      break;
    case gBIT_ANDTC_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op1);
      break;
    case gBIT_NOT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].bit_not_expr.expr1.op0);
      break;
    case gTRUTH_ANDIF_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_andif_expr.op0);
      do_edge(g.nodeBuffer[nodenum].truth_andif_expr.op1);
      break;
    case gTRUTH_ORIF_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_orif_expr.op0);
      do_edge(g.nodeBuffer[nodenum].truth_orif_expr.op1);
      break;
    case gTRUTH_AND_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_and_expr.op0);
      do_edge(g.nodeBuffer[nodenum].truth_and_expr.op1);
      break;
    case gTRUTH_OR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_or_expr.op0);
      do_edge(g.nodeBuffer[nodenum].truth_or_expr.op1);
      break;
    case gTRUTH_XOR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_xor_expr.op0);
      do_edge(g.nodeBuffer[nodenum].truth_xor_expr.op1);
      break;
    case gTRUTH_NOT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].truth_not_expr.op0);
      break;
    case gLT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].lt_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].lt_expr.expr_ineq.op1);
      break;
    case gLE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].le_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].le_expr.expr_ineq.op1);
      break;
    case gGT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].gt_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].gt_expr.expr_ineq.op1);
      break;
    case gGE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ge_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].ge_expr.expr_ineq.op1);
      break;
    case gEQ_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].eq_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].eq_expr.expr_ineq.op1);
      break;
    case gNE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ne_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].ne_expr.expr_ineq.op1);
      break;
    case gUNORDERED_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op1);
      break;
    case gORDERED_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op1);
      break;
    case gUNLT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op1);
      break;
    case gUNLE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].unle_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].unle_expr.expr_ineq.op1);
      break;
    case gUNGT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op1);
      break;
    case gUNGE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].unge_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].unge_expr.expr_ineq.op1);
      break;
    case gUNEQ_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op1);
      break;
    case gIN_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].in_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].in_expr.expr2.op1);
      break;
    case gSET_LE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op0);
      do_edge(g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op1);
      break;
    case gCARD_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].card_expr.expr1.op0);
      break;
    case gRANGE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].range_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].range_expr.expr2.op1);
      break;
    case gCONVERT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].convert_expr.expr1.op0);
      break;
    case gNOP_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].nop_expr.expr1.op0);
      break;
    case gNON_LVALUE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].non_lvalue_expr.expr1.op0);
      break;
    case gSAVE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].save_expr.op0);
      break;
    case gUNSAVE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gRTL_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gADDR_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].addr_expr.op0);
      break;
    case gREFERENCE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gENTRY_VALUE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gCOMPLEX_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].complex_expr.expr2.op0);
      do_edge(g.nodeBuffer[nodenum].complex_expr.expr2.op1);
      break;
    case gCONJ_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].conj_expr.expr1.op0);
      break;
    case gREALPART_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].realpart_expr.expr1.op0);
      break;
    case gPREDECREMENT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].predecrement_expr.op0);
      do_edge(g.nodeBuffer[nodenum].predecrement_expr.op1);
      break;
    case gPREINCREMENT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].preincrement_expr.op0);
      do_edge(g.nodeBuffer[nodenum].preincrement_expr.op1);
      break;
    case gPOSTDECREMENT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].postdecrement_expr.op0);
      do_edge(g.nodeBuffer[nodenum].postdecrement_expr.op1);
      break;
    case gPOSTINCREMENT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].postincrement_expr.op0);
      do_edge(g.nodeBuffer[nodenum].postincrement_expr.op1);
      break;
    case gVA_ARG_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].va_arg_expr.op0);
      break;
    case gTRY_CATCH_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].try_catch_expr.op0);
      do_edge(g.nodeBuffer[nodenum].try_catch_expr.op1);
      break;
    case gTRY_FINALLY_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].try_finally_expr.op0);
      do_edge(g.nodeBuffer[nodenum].try_finally_expr.op1);
      break;
    case gGOTO_SUBROUTINE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].goto_subroutine_expr.op0);
      do_edge(g.nodeBuffer[nodenum].goto_subroutine_expr.op1);
      break;
    case gPOPDHC_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gPOPDCC_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gLABEL_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gGOTO_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gRETURN_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gEXIT_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].exit_expr.cond);
      break;
    case gLOOP_EXPR:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].loop_expr.body);
      break;
    case gLABELED_BLOCK_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gEXPR_WITH_FILE_LOCATION:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].expr_with_file_location.expr2);
      break;
    case gSWITCH_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].switch_expr.op0);
      do_edge(g.nodeBuffer[nodenum].switch_expr.op1);
      break;
    case gSIZEOF_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].sizeof_expr.op0);
      break;
    case gARROW_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].arrow_expr.op0);
      break;
    case gALIGNOF_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].alignof_expr.op0);
      break;
    case gNEW_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].new_expr.op0);
      break;
    case gVEC_NEW_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].vec_new_expr.op0);
      break;
    case gDELETE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].delete_expr.op0);
      break;
    case gVEC_DELETE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].vec_delete_expr.op0);
      break;
    case gTYPE_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].type_expr.op0);
      break;
    case gAGGR_INIT_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].aggr_init_expr.fn);
      do_edge(g.nodeBuffer[nodenum].aggr_init_expr.args);
      do_edge(g.nodeBuffer[nodenum].aggr_init_expr.decl);
      break;
    case gEMPTY_CLASS_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gTEMPLATE_ID_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].template_id_expr.op0);
      do_edge(g.nodeBuffer[nodenum].template_id_expr.op1);
      break;
    case gLOOKUP_EXPR:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].lookup_expr.op0);
      break;
    case gFUNCTION_NAME:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gSUBOBJECT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].subobject.clnp);
      do_edge(g.nodeBuffer[nodenum].subobject.next);
      break;
    case gCTOR_STMT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ctor_stmt.next);
      break;
    case gCTOR_INITIALIZER:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ctor_initializer.op0);
      do_edge(g.nodeBuffer[nodenum].ctor_initializer.op1);
      break;
    case gRETURN_INIT:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].return_init.op0);
      do_edge(g.nodeBuffer[nodenum].return_init.op1);
      break;
    case gTAG_DEFN:
      do_expr(&g.nodeBuffer[nodenum].expr);
      break;
    case gIDENTITY_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].identity_conv.op0);
      break;
    case gLVALUE_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].lvalue_conv.op0);
      break;
    case gQUAL_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].qual_conv.op0);
      break;
    case gSTD_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].std_conv.op0);
      break;
    case gPTR_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ptr_conv.op0);
      break;
    case gPMEM_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].pmem_conv.op0);
      break;
    case gBASE_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].base_conv.op0);
      break;
    case gREF_BIND:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ref_bind.op0);
      break;
    case gUSER_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].user_conv.op0);
      break;
    case gAMBIG_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].ambig_conv.op0);
      break;
    case gRVALUE_CONV:
      do_expr(&g.nodeBuffer[nodenum].expr);
      do_edge(g.nodeBuffer[nodenum].rvalue_conv.op0);
      break;
    case gOP_IDENTIFIER:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gIDENTIFIER_NODE:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gINTEGER_CST:
      do_cst(&g.nodeBuffer[nodenum].gcc_cst);
      break;
    case gREAL_CST:
      do_cst(&g.nodeBuffer[nodenum].gcc_cst);
      break;
    case gPTRMEM_CST:
      do_cst(&g.nodeBuffer[nodenum].gcc_cst);
      do_edge(g.nodeBuffer[nodenum].ptrmem_cst.clas);
      do_edge(g.nodeBuffer[nodenum].ptrmem_cst.mbr);
      break;
    case gCOMPLEX_CST:
      do_cst(&g.nodeBuffer[nodenum].gcc_cst);
      break;
    case gSTRING_CST:
      do_cst(&g.nodeBuffer[nodenum].gcc_cst);
      break;
    case gTREE_LIST:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].tree_list.purpose);
      do_edge(g.nodeBuffer[nodenum].tree_list.value);
      do_edge(g.nodeBuffer[nodenum].tree_list.chan);
      break;
    case gTREE_VEC:
      if (g.nodeBuffer[nodenum].tree_vec.binfo){
	do_common(&g.nodeBuffer[nodenum].common);
	do_edge(g.nodeBuffer[nodenum].tree_vec.inherit);
	do_edge(g.nodeBuffer[nodenum].tree_vec.type);
	do_edge(g.nodeBuffer[nodenum].tree_vec.base);
      } else {
	do_common(&g.nodeBuffer[nodenum].common);
      }
      break;
    case gERROR_MARK:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gDEFAULT_ARG:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gCPLUS_BINDING:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
    case gOVERLOAD:
      do_common(&g.nodeBuffer[nodenum].common);
      do_edge(g.nodeBuffer[nodenum].overload.crnt);
      do_edge(g.nodeBuffer[nodenum].overload.chan);
      break;
    case gWRAPPER:
      do_common(&g.nodeBuffer[nodenum].common);
      break;
      /* datrix types */
    case dC_ASG_ND:
      datrix_asg_nd(& g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_IDENTIFIER:
      datrix_identifier(& g.nodeBuffer[nodenum].dc_identifier);
      break;
    case dC_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_BUILT_IN_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_AGGR_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_aggr_type.members);
      do_edge(g.nodeBuffer[nodenum].dc_aggr_type.inheritance);
      break;
    case dC_ENUM_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_enum_type.values);
      break;
    case dC_TEMPL_PARM_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_GENERIC_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_generic_type.data_functs);
      do_edge(g.nodeBuffer[nodenum].dc_generic_type.formals);
      break;
    case dC_GENERATED_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_generated_type.actuals);
      break;
    case dC_ALIAS_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_alias_type.original);
      break;
    case dC_FORWARD_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_ARRAY_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
   case dC_PTR_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_REF_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_FCT_TYPE:
      datrix_type(& g.nodeBuffer[nodenum].dc_type);
      do_edge(g.nodeBuffer[nodenum].dc_fct_type.formalParams);
      break;
    case dC_SCOPE:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_GLB:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_COMPIL:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_FILE:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_PACKAGE:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_NAME_SPACE:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_STATIC_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_TRY_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_CATCH_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_FINALLY_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SYNCHRONIZED_BLOCK:
      datrix_scope(& g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_OBJECT:
      datrix_object(& g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FORMAL_FCT_PARAM:
      datrix_object(& g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FORMAL_CATCH_PARAM:
      datrix_object(& g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FUNCTION:
      datrix_identifier(& g.nodeBuffer[nodenum].dc_identifier);
      do_edge(g.nodeBuffer[nodenum].dc_function.formal_fct);
      do_edge(g.nodeBuffer[nodenum].dc_function.exception);
      do_edge(g.nodeBuffer[nodenum].dc_function.block);
      break;
    case dC_LABEL:
      datrix_label(& g.nodeBuffer[nodenum].dc_label);
      break;
    case dC_CASE_LABEL:
      datrix_label(& g.nodeBuffer[nodenum].dc_label);
      do_edge(g.nodeBuffer[nodenum].dc_case_label.value);
      break;
    case dC_DEFAULT_LABEL:
      datrix_label(& g.nodeBuffer[nodenum].dc_label);
      break;
    case dC_ENUMERATOR:
      datrix_identifier(& g.nodeBuffer[nodenum].dc_identifier);
      do_edge(g.nodeBuffer[nodenum].dc_enumerator.value);
      break;
    case dC_EXPRESSION:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_EMPTY_EXPR:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_LITERAL:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_literal.type);
      break;
    case dC_SYNTAX_ERROR:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_NAME_REF:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_OPERATOR:
      datrix_operator(& g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_UNARY_OPERATOR:
      datrix_operator(& g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_BINARY_OPERATOR:
      datrix_operator(& g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_QUESTION_OPERATOR:
      datrix_operator(& g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_INIT_LIST:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_init_list.init);
      break;
    case dC_ACTUAL_FCT_PARAM_LIST:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_actual_fct_param_list.actuals);
      break;
    case dC_ACTUAL_TEMPL_PARAM_LIST:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_actual_templ_param_list.actuals);
      break;
    case dC_CREATION_EXPR:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_creation_expr.init);
      break;
    case dC_DESTROY_EXPR:
      datrix_expression(& g.nodeBuffer[nodenum].dc_expression);
      do_edge(g.nodeBuffer[nodenum].dc_destroy_expr.what);
      do_edge(g.nodeBuffer[nodenum].dc_destroy_expr.array);
      break;
    case dC_CNTRL_STMT:
      datrix_ctrl_stmt(& g.nodeBuffer[nodenum].dc_ctrl_stmt);
      break;
    case dC_SELECTION:
      datrix_ctrl_stmt(& g.nodeBuffer[nodenum].dc_ctrl_stmt);
      break;
    case dC_IF:
      datrix_ctrl_stmt(& g.nodeBuffer[nodenum].dc_ctrl_stmt);
      do_edge(g.nodeBuffer[nodenum].dc_if.cond);
      do_edge(g.nodeBuffer[nodenum].dc_if.true_block);
      do_edge(g.nodeBuffer[nodenum].dc_if.false_block);
      break;
    case dC_SWITCH:
      datrix_ctrl_stmt(& g.nodeBuffer[nodenum].dc_ctrl_stmt);
      do_edge(g.nodeBuffer[nodenum].dc_switch.cond);
      do_edge(g.nodeBuffer[nodenum].dc_switch.body);
      break;
    case dC_JUMP:
      datrix_jump(& g.nodeBuffer[nodenum].dc_jump);
      break;
    case dC_GOTO:
      datrix_jump(& g.nodeBuffer[nodenum].dc_jump);
      do_edge(g.nodeBuffer[nodenum].dc_goto.dest);
      break;
    case dC_RETURN:
      datrix_jump(& g.nodeBuffer[nodenum].dc_jump);
      do_edge(g.nodeBuffer[nodenum].dc_return.value);
      break;
    case dC_BREAK:
      datrix_jump(& g.nodeBuffer[nodenum].dc_jump);
      do_edge(g.nodeBuffer[nodenum].dc_break.dest);
      break;
    case dC_CONTINUE:
      datrix_jump(& g.nodeBuffer[nodenum].dc_jump);
      do_edge(g.nodeBuffer[nodenum].dc_continue.dest);
      break;
    case dC_LOOP:
      datrix_loop(& g.nodeBuffer[nodenum].dc_loop);
      break;
    case dC_DO_WHILE_LOOP:
      datrix_loop(& g.nodeBuffer[nodenum].dc_loop);
      do_edge(g.nodeBuffer[nodenum].dc_do_while_loop.cond);
      do_edge(g.nodeBuffer[nodenum].dc_do_while_loop.block);
      break;
    case dC_WHILE_DO_LOOP:
      datrix_loop(& g.nodeBuffer[nodenum].dc_loop);
      do_edge(g.nodeBuffer[nodenum].dc_while_do_loop.cond);
      do_edge(g.nodeBuffer[nodenum].dc_while_do_loop.block);
      break;
    case dC_FOR_LOOP:
      datrix_loop(& g.nodeBuffer[nodenum].dc_loop);
      do_edge(g.nodeBuffer[nodenum].dc_for_loop.cond);
      do_edge(g.nodeBuffer[nodenum].dc_for_loop.block);
      do_edge(g.nodeBuffer[nodenum].dc_for_loop.init);
      do_edge(g.nodeBuffer[nodenum].dc_for_loop.incr);
      break;
    case dC_SQL_STMT:
      datrix_asg_nd(& g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_COMMENT:
      datrix_comment(& g.nodeBuffer[nodenum].dc_comment);
      break;
    case dC_JAVA_DOC_COMMENT:
      datrix_comment(& g.nodeBuffer[nodenum].dc_comment);
      break;
    case dC_THROW_SPEC:
      datrix_asg_nd(&g.nodeBuffer[nodenum].dc_asg_nd);
      do_edge(g.nodeBuffer[nodenum].dc_throw_spec.except);
      break;
    case dC_ASM_BLOCK:
      datrix_asg_nd(&g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_ARC:
      datrix_arc(&g.nodeBuffer[nodenum].dc_arc);
      break;
    case dC_ARC_REFERENCE:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_INSTANCE:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_INHERITANCE:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
    case dC_IMPLEMENTS:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_FRIEND:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_DEF_LOC:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_REFERS_TO:
      datrix_arc_reference(&g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_SYNTAX:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_SON:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_TRUE:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FALSE:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_OPD:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_IMPORT:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_COND:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FOR_INIT:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FOR_INCR:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_INIT_VAL:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_BIT_DIM:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_ARRAY_DIM:
      datrix_arc_syntax(&g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    default:
      /* do nothing */
      break;
  }
}

void datrix_asg_nd(t_dc_asg_nd *nd){
  do_common(&(nd->common));
}

void datrix_identifier(t_dc_identifier *ident){
  datrix_asg_nd(&(ident->dc_asg_nd));
  do_edge(ident->instance);
  do_edge(ident->defLoc);
  do_edge(ident->arcFriend);
  do_edge(ident->array_dim);
}

void datrix_loop(t_dc_loop *loop){
  datrix_ctrl_stmt(&(loop->ctrl_stmt));
}

void datrix_ctrl_stmt(t_dc_ctrl_stmt *cntrl){
  datrix_asg_nd(&(cntrl->dc_asg_nd));
}

void datrix_comment(t_dc_comment *comment){
  datrix_asg_nd(&(comment->dc_asg_nd));
}

void datrix_jump(t_dc_jump *jump) {
  datrix_ctrl_stmt(&(jump->ctrl_stmt));
}

void datrix_expression(t_dc_expression *expr){
  datrix_asg_nd(&(expr->dc_asg_nd));
  do_edge(expr->refers);
}

void datrix_operator(t_dc_operator *operator){
  datrix_expression(&(operator->expression));
  do_edge(operator->ops);
}

void datrix_type(t_dc_type *type){
  datrix_identifier(&(type->dc_identifier));
  do_edge(type->implements);
}

void datrix_object(t_dc_object *object){
  datrix_identifier(&(object->dc_identifier));
  do_edge(object->bitfield_dim);
  do_edge(object->initial_val);
}

void datrix_label(t_dc_label *label){
  datrix_identifier(&(label->dc_identifier));
}

void datrix_scope(t_dc_scope *scope){
  datrix_identifier(&(scope->dc_identifier));
  do_edge(scope->contents);
}

void datrix_arc(t_dc_arc  *dc_arc){
  do_common(&(dc_arc->common));
  do_edge(dc_arc->next);
  do_edge(dc_arc->src); /* technically we shouldn't have to do this one */
  do_edge(dc_arc->dest);
}

void datrix_arc_reference(t_dc_arc_reference  *dc_arc_reference){
  datrix_arc(&(dc_arc_reference->arc));
}

void datrix_arc_syntax(t_dc_arc_syntax  *dc_arc_syntax){
  datrix_arc(&(dc_arc_syntax->arc));
}

void do_common(gd_graph_common * common){
  do_edge(common->other_node);
  /* mark the node as visited */
  common->mark_flag1 = 1;
}

/* returns 1 if the pointers are to be explored */
/* this also handles the special case of decl nodes */
/* on the main spine that we are attempting to remove */
/* do common is only called for nodes that we are officially
 * visiting, since it marks the nodes as a keeper */

int do_decl(t_gcc_decl *decl, int byChan){
  if (byChan && decl->common.mark_flag2) {
    /* special case for spine by chan */
    queue_add(decl->chan,1);
    if (decl->srcf != internal){
      /* user declared global entity */
      do_common(&(decl->common)); /* keep the node */
      queue_add(decl->name,0); /* and all of the contents */
      queue_add(decl->mngl,0);
      queue_add(decl->type,0);
      queue_add(decl->scpe,0);
      return 1;
    }
    return 0; /* don't visit other contents of decl node */
  } else {
    /* we reached the decl from within */
    do_common(&(decl->common)); /* keep the node */
    queue_add(decl->name,0); /* and all of the contents */
    queue_add(decl->mngl,0);
    queue_add(decl->type,0);
    if (!decl->common.mark_flag2){
      /* follow internal chans, not spine chans */
      queue_add(decl->chan,0);
    } else {
      queue_add(decl->chan,1);
    }
    queue_add(decl->scpe,0);
    return 1;
  }
}

void do_edge(node_ptr nextNode){
  queue_add(nextNode,0);
}

void do_type(t_gcc_type *type){
  do_common(&(type->common)); /* keep the node */
  queue_add(type->name,0);
  queue_add(type->size,0);
  queue_add(type->unql,0);
}

void do_cst(t_gcc_const *cst){
  do_common(&(cst->common)); /* keep the node */
  queue_add(cst->type,0);
}

void do_expr(t_gcc_expr *expr){
  do_common(&(expr->common)); /* keep the node */
  queue_add(expr->type,0);
}
