// Replace t_for_stmts by t_dc_for_loop and associated arcs.

/* AJM 2001 April 2 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"
#include "gd_next.h"

int main (int argc, char *argv [])
{
    void do_for (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_for (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// Replace t_for_stmt by t_dc_for_loop:
//	next -> other_node
//	init -> new forinit arcs
//	cond -> new cond arc
//	body -> new arcson
//	inc_expr -> new forincr

// Note that in gcc, the init ptr always points to a sequence of statements,
// the first of which is a scope_stmt, and the remainder are either decls or
// expressions.  The init of `for (x = 5, y = 9; ...' compiles to -two- statements.
// This is preserved.

void do_for (gd_graph *g)
{
    node_ptr ptr;

    // For handling the inits chain and making FOR_INIT arcs.
    node_ptr init_ptr;		// ptr to an element of the inits
    node_ptr *cur_arc;		// pointer to ptr of current arc

    // For handling the body chain, finding the actual body block.
    node_ptr stmt_ptr;		// ptr to an element of the body chain
    node_ptr next_ptr;		// ptr to next in the body chain

    // For handling the increment, if any
    node_ptr incr_ptr;		// ptr to the increment expression

    // New nodes
    gd_graph_node new_for;
    gd_graph_node new_arc_init;
    gd_graph_node new_arc_cond;
    gd_graph_node new_arc_incr;
    gd_graph_node new_arc_body;

    for (ptr = 0; ptr < g-> curNodePos; ++ptr)
	if (g-> nodeBuffer [ptr]. common. type == gFOR_STMT)
	{
	    // Make a new for loop node.
	    new_for. common. type = dC_FOR_LOOP;
	    new_for. common. other_node = g-> nodeBuffer [ptr]. for_stmt. next;

	    // 1) Walk the initializers list.
	    init_ptr = g-> nodeBuffer [ptr]. for_stmt. init;
	    cur_arc = &new_for. dc_for_loop. init;
	    *cur_arc = -1;

	    while (init_ptr >= 0 && init_ptr < g-> curNodePos)
	    {
		if (
			g-> nodeBuffer [init_ptr]. common. type == gEXPR_STMT
		    ||	g-> nodeBuffer [init_ptr]. common. type == gDECL_STMT
		    )
		{
		    // Make an edge to discovered thing.
		    new_arc_init. common. type = dC_ARC_FOR_INIT;
		    new_arc_init. common. other_node = -1;
		    new_arc_init. dc_arc. ordinal = 0;	// to be set by a later task
		    new_arc_init. dc_arc. src = ptr;
		    new_arc_init. dc_arc. dest = init_ptr;
		    new_arc_init. dc_arc. next = -1;	// to be changed on the next cycle

		    // Save the edge.
		    *cur_arc = newNode (g);
		    fprintf (stderr, "new node %d for init edge\n", *cur_arc);
		    g-> nodeBuffer [*cur_arc] = new_arc_init;

		    // Point at the ptr for the next arc, if any.
		    cur_arc = &g-> nodeBuffer [*cur_arc]. dc_arc. next;
		}
		
		// Step to the next node in the init chain.
		switch (g-> nodeBuffer [init_ptr]. common. type)
		{
		case gEXPR_STMT:
		    init_ptr = g-> nodeBuffer [init_ptr]. expr_stmt. next;
		    break;
		case gDECL_STMT:
		    init_ptr = g-> nodeBuffer [init_ptr]. decl_stmt. next;
		    break;
		case gSCOPE_STMT:
		    init_ptr = g-> nodeBuffer [init_ptr]. scope_stmt. next;
		    break;
		default:
		    fprintf (stderr,
			    "gd_for: unexpected type for node %d in initializer of for-loop %d\n",
			    init_ptr,
			    ptr
			);
		    init_ptr = -1;
		}
	    }

	    // 2) Make a new condition arc.
	    new_arc_cond. common. type = dC_ARC_COND;
	    new_arc_cond. common. other_node = -1;
	    new_arc_cond. dc_arc. ordinal = 0;
	    new_arc_cond. dc_arc. src = ptr;

	    // The cond is (as usual) a tree_list, whose value is the thing we need.
	    new_arc_cond. dc_arc. dest = g-> nodeBuffer [g-> nodeBuffer [ptr]. for_stmt. cond]. tree_list. value;
	    new_arc_cond. dc_arc. next = -1;
	    
	    // Save it.
	    new_for. dc_for_loop. cond = newNode (g);
	    g-> nodeBuffer [new_for. dc_for_loop. cond] = new_arc_cond;

	    // 3) Make a new body arc, by skipping scope statements
	    // until we find something else.
	    stmt_ptr = g-> nodeBuffer [ptr]. for_stmt. body;
	    while (
		    0 <= stmt_ptr
		&&  stmt_ptr <= g-> curNodePos
		&&  g-> nodeBuffer [stmt_ptr]. common. type == gSCOPE_STMT
		)
	    {
		next_ptr = g-> nodeBuffer [stmt_ptr]. scope_stmt. next;
		// Detach it for easier garbage collection.
		g-> nodeBuffer [stmt_ptr]. scope_stmt. next = -1;
		stmt_ptr = next_ptr;
	    }
	    
	    // If we found something, it's the body now.
	    if (
		    0 <= stmt_ptr
		&&  stmt_ptr <= g-> curNodePos
		)
	    {
		// Detach this from the rest of the body junk, if any.
		setNext (g, stmt_ptr, -1);

		new_arc_body. common. type = dC_ARC_SON;
		new_arc_body. common. other_node = -1;
		new_arc_body. dc_arc. ordinal = 0;
		new_arc_body. dc_arc. src = ptr;
		new_arc_body. dc_arc. dest = stmt_ptr;
		new_arc_body. dc_arc. next = -1;

		new_for. dc_for_loop. block = newNode (g);
		g-> nodeBuffer [new_for. dc_for_loop. block] = new_arc_body;
	    }
	    else
		new_for. dc_for_loop. block = -1;

	    // 4) Make an incr edge, if needed.
	    incr_ptr = g-> nodeBuffer [ptr]. for_stmt. inc_expr;
	    if (0 <= incr_ptr && incr_ptr <= g-> curNodePos)
	    {
		new_arc_incr. common. type = dC_ARC_FOR_INCR;
		new_arc_incr. common. other_node = -1;
		new_arc_incr. dc_arc. ordinal = 0;
		new_arc_incr. dc_arc. src = ptr;
		new_arc_incr. dc_arc. dest = incr_ptr;
		new_arc_incr. dc_arc. next = -1;

		new_for. dc_for_loop. incr = newNode (g);
		g-> nodeBuffer [new_for. dc_for_loop. incr] = new_arc_incr;
	    }
	    else
		new_for. dc_for_loop. incr = -1;

	    // Save the new for node back.
	    g-> nodeBuffer [ptr] = new_for;
	}
}
