// Labels are represented in gcc by t_label_decls, and t_case_labels.
// Replace them with dc_labels, and dc_case_labels and dc_default_labels,
// respectively.

/* AJM 2001 April 2 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_labels (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_labels (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// Replace t_label_decl nodes by t_dc_label nodes.
// Replace t_case_label nodes having a low ptr by t_dc_case_label nodes.
// Replace t_case_label nodes having no low ptr by t_dc_default_label nodes.

// In t_integer_cst, the fields high and low are the top and bottom of a 64-bit value.
// BUT: in t_case_labels, they're ptrs to actual literals.    Apparently they are for
// representing a range (as in Pascal: case 3 .. 5).  Since C++ has no case ranges, only
// the low ptr is interesting.

void do_labels (gd_graph *g)
{
    node_ptr ptr;

    gd_graph_node new_label;
    gd_graph_node new_arc_son;

    for (ptr = 0; ptr < g-> curNodePos; ++ptr)
	if (g-> nodeBuffer [ptr]. common. type == gLABEL_DECL)
	{
	    // Make a new label node.
	    // (This label decl is normally enclosed in a label_stmt, so there is no next
	    // ptr here, not any need to try to set other_node to anything special.)
	    new_label. common. type = dC_LABEL;
	    new_label. common. other_node = -1;
	    new_label. dc_identifier. name = g-> nodeBuffer [g-> nodeBuffer [ptr]. decl. name]. identifier_node. strg;
	    new_label. dc_identifier. visb = visb_unk;
	    new_label. dc_identifier. instance = -1;
	    new_label. dc_identifier. defLoc = -1;
	    new_label. dc_identifier. arcFriend = -1;
	    new_label. dc_identifier. array_dim = -1;

	    g-> nodeBuffer [ptr] = new_label;
	}
	else if (g-> nodeBuffer [ptr]. common. type == gCASE_LABEL)
	{
	    // What kind of new label node?
	    if (g-> nodeBuffer [ptr]. case_label. low == -1)

		// Now low-ptr, so it's a default label.
		new_label. common. type = dC_DEFAULT_LABEL;

	    else
	    {
		// Has low-ptr, so it's a case label.
		new_label. common. type = dC_CASE_LABEL;
		new_label. dc_case_label. value = newNode (g);	// for arcson edge to label value

		// Construct new son edge for case label value.
		new_arc_son. common. type = dC_ARC_SON;
		new_arc_son. common. other_node = -1;
		new_arc_son. dc_arc. ordinal = 0;
		new_arc_son. dc_arc. src = ptr;
		new_arc_son. dc_arc. dest = g-> nodeBuffer [ptr]. case_label. low;
		new_arc_son. dc_arc. next = -1;

		// Save new son edge.
		g-> nodeBuffer [new_label. dc_case_label. value] = new_arc_son;
	    }

	    // Construct the rest of the label node.
	    new_label. common. other_node = g-> nodeBuffer [ptr]. case_label. next;
	    new_label. dc_identifier. name = -1; // anonymous!
	    new_label. dc_identifier. visb = visb_unk;
	    new_label. dc_identifier. instance = -1;
	    new_label. dc_identifier. defLoc = -1;
	    new_label. dc_identifier. arcFriend = -1;
	    new_label. dc_identifier. array_dim = -1;

	    // Save it.
	    g-> nodeBuffer [ptr] = new_label;
	}
}
