// Make C_OBJECT nodes.  An Object in Datrix is anything that can be assigned to
// (or could be if it were not const), which means:
//	local variables in methods and functions
//	member variables in classes
//	formal parameters (of types, functions, and handlers)
//	bit fields
// It excludes enumerated constants, of course.
// This program only handles concrete C_OBJECT nodes, the subclasses
// (formal parameters) are handled elsewhere.

/* AJM 2001 Mar 14 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_objects (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_objects (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

void do_objects (gd_graph *g)
{
    node_ptr ptr;			// Ptr to node which is to be converted to object.
    gd_graph_node *obj_node_p;		// Pointer to node which is object

    gd_graph_node new_obj_node;		// New Datrix node for object.
    gd_graph_node new_edge;		// New Datrix edge for various purposes.

    // Do all the objects.
    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
    {
	obj_node_p = &(g-> nodeBuffer [ptr]);

	// Converting variables and fields.
	if (
		obj_node_p-> common. type == gVAR_DECL
	    ||	obj_node_p-> common. type == gFIELD_DECL
	)
	{
	    // gd_graph_common
	    new_obj_node. common. type = dC_OBJECT;
	    new_obj_node. common. other_node = -1;

	    // t_dc_asg_nd
	    new_obj_node. dc_asg_nd. file = g-> nodeBuffer [obj_node_p-> type. name]. decl. srcf;
	    new_obj_node. dc_asg_nd. line = g-> nodeBuffer [obj_node_p-> type. name]. decl. srcl;

	    // t_dc_identifier
	    new_obj_node. dc_identifier. name = g-> nodeBuffer [obj_node_p-> const_decl. decl. name]. identifier_node. strg;
	    new_obj_node. dc_identifier. visb =
		    obj_node_p-> common. public_flag? visb_pub:
		    obj_node_p-> common. private_flag? visb_pri:
		    obj_node_p-> common. protected_flag? visb_pro:
		    visb_unk
		;
	    new_obj_node. dc_identifier. instance = newNode (g);

	    new_edge. common. type = dC_INSTANCE;
	    new_edge. common. other_node = -1;
	    new_edge. dc_arc. ordinal = 0;
	    new_edge. dc_arc. src = ptr;
	    new_edge. dc_arc. dest = obj_node_p-> decl. type;
	    new_edge. dc_arc. next = -1;

	    g-> nodeBuffer [new_obj_node. dc_identifier. instance] = new_edge;

	    new_obj_node. dc_identifier. defLoc = -1;
	    new_obj_node. dc_identifier. arcFriend = -1;
	    new_obj_node. dc_identifier. array_dim = -1;

	    // t_dc_object
	    new_obj_node. dc_object. static_flag = obj_node_p-> common. static_flag;
	    new_obj_node. dc_object. extern_flag = 0;		// TODO
	    new_obj_node. dc_object. const_flag = obj_node_p-> common. constant_flag;

	    new_obj_node. dc_object. bitfield_dim = -1;
	    new_obj_node. dc_object. initial_val = -1;

	    // Perhaps there's a bit field size.
	    if (obj_node_p-> common. type == gFIELD_DECL)
	    {
		new_obj_node. dc_object. bitfield_dim = newNode (g);

		if (obj_node_p -> field_decl.bitfield){

		  new_edge. common. type = dC_ARC_BIT_DIM;
		  new_edge. dc_arc. ordinal = 0;
		  new_edge. dc_arc. src = ptr;
		  new_edge. dc_arc. dest = obj_node_p-> field_decl. size;
		  new_edge. dc_arc. next = -1;

		  g-> nodeBuffer [new_obj_node. dc_object. bitfield_dim] = new_edge;
		}
	    }

	    // Perhaps there's an initial value.
	    if (obj_node_p-> common. type == gVAR_DECL && obj_node_p-> var_decl. init != -1)
	    {
		new_obj_node. dc_object. initial_val = newNode (g);

		new_edge. common. type = dC_ARC_INIT_VAL;
		new_edge. dc_arc. ordinal = 0;
		new_edge. dc_arc. src = ptr;
		new_edge. dc_arc. dest = obj_node_p-> var_decl. init;
		new_edge. dc_arc. next = -1;

		g-> nodeBuffer [new_obj_node. dc_object. initial_val] = new_edge;
	    }
	    
	    // save it back
	    *obj_node_p = new_obj_node;
	}
    }
}
