// Statements (expr_stmts, label_stmts, decl_stmts) from gcc are not represented in Datrix
// at all: the thing the statement encloses (an expression, a label, a object declaration) is
// simply supposed to be an arcson of the enclosing scope.

// This program skips edges to expr_stmts, label_stmts, and decl_stmts, in favour of the thing
// they enclose.

/* AJM 2001 Apr 10 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

// Defined in gd_walkers.c
void walk_ptrs (gd_graph *, void (*) (gd_graph *, node_ptr, node_ptr *, int));

// Defined here.
void cvt_stmt (gd_graph *, node_ptr, node_ptr *, int);

int main (int argc, char *argv [])
{
    void do_expr_stmts (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    walk_ptrs (&g, cvt_stmt);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

void cvt_stmt (gd_graph *g, node_ptr p, node_ptr *stmt_ptr, int n)
{
    // Skip any ptrs to other_node.
    if (n == 0)
	return;
    
    // Ignore nonptrs.
    if (0 <= *stmt_ptr && *stmt_ptr < g-> curNodePos); else return;

    // Redirect ptrs if they pt to skippable nodes.
    if (g-> nodeBuffer [*stmt_ptr]. common. type == gEXPR_STMT)
	*stmt_ptr = g-> nodeBuffer [*stmt_ptr]. expr_stmt. texpr;
    else
    if (g-> nodeBuffer [*stmt_ptr]. common. type == gDECL_STMT)
	*stmt_ptr = g-> nodeBuffer [*stmt_ptr]. decl_stmt. decl;
    else
    if (g-> nodeBuffer [*stmt_ptr]. common. type == gLABEL_STMT)
	*stmt_ptr = g-> nodeBuffer [*stmt_ptr]. label_stmt. label;
}
