// Type Declarations (type_decl) are not used in the Datrix model: we use the types instead.
// This program skips edges to type_decls in favour of the types they decl, which by this point
// ought to have been converted into datrix types.

/* AJM 2001 Apr 10 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

// Defined in gd_walkers.c
void walk_ptrs (gd_graph *, void (*) (gd_graph *, node_ptr, node_ptr *, int));

// Defined below...
void cvt_type_decl (gd_graph *, node_ptr, node_ptr *, int);

int main (int argc, char *argv [])
{
    void do_type_decls (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    walk_ptrs (&g, cvt_type_decl);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

void cvt_type_decl (gd_graph *g, node_ptr p, node_ptr *decl_ptr, int n)
{
    // Skip any ptrs to other_node.
    if (n == 0)
	return;
    
    // Ignore nonptrs.
    if (0 <= *decl_ptr && *decl_ptr < g-> curNodePos); else return;

    // Redirect ptrs to type_decls to pt to their types.
    if (g-> nodeBuffer [*decl_ptr]. common. type == gTYPE_DECL)
	*decl_ptr = g-> nodeBuffer [*decl_ptr]. type_decl. decl. type;
}
