#include <gd_graph.h>
#include "gd_utils.h"

// This procedure walks an entire graph G, dispatching a given function F
// once for every node_ptr field F of every node N.  The function F is passed
//	* pointer to G
//	* ptr of N
//	* pointer to F
//	* the ordinal of F within all the node_ptr fields in N

// It's reentrant, so you can rewalk the graph while walking it, if necessary.

void walk_ptrs (gd_graph *g, void *f (gd_graph *, node_ptr, node_ptr *, int))
{
    int i;
    for (i = 0; i < g-> curNodePos; ++ i)
	switch (g-> nodeBuffer [i]. common. type)
	{
	case gERROR_MARK: w_error_mark (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gIDENTIFIER_NODE: w_identifier_node (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gOP_IDENTIFIER: w_op_identifier (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTREE_LIST: w_tree_list (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTREE_VEC: w_tree_vec (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBLOCK: w_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVOID_TYPE: w_void_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gINTEGER_TYPE: w_integer_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREAL_TYPE: w_real_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPLEX_TYPE: w_complex_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVECTOR_TYPE: w_vector_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gENUMERAL_TYPE: w_enumeral_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBOOLEAN_TYPE: w_boolean_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCHAR_TYPE: w_char_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPOINTER_TYPE: w_pointer_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gOFFSET_TYPE: w_offset_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREFERENCE_TYPE: w_reference_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMETHOD_TYPE: w_method_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFILE_TYPE: w_file_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gARRAY_TYPE: w_array_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSET_TYPE: w_set_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRECORD_TYPE: w_record_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNION_TYPE: w_union_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gQUAL_UNION_TYPE: w_qual_union_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFUNCTION_TYPE: w_function_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLANG_TYPE: w_lang_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gINTEGER_CST: w_integer_cst (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREAL_CST: w_real_cst (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPLEX_CST: w_complex_cst (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSTRING_CST: w_string_cst (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFUNCTION_DECL: w_function_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLABEL_DECL: w_label_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCONST_DECL: w_const_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTYPE_DECL: w_type_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVAR_DECL: w_var_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPARM_DECL: w_parm_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRESULT_DECL: w_result_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFIELD_DECL: w_field_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNAMESPACE_DECL: w_namespace_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPONENT_REF: w_component_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_FIELD_REF: w_bit_field_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gINDIRECT_REF: w_indirect_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBUFFER_REF: w_buffer_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gARRAY_REF: w_array_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCONSTRUCTOR: w_constructor (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPOUND_EXPR: w_compound_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMODIFY_EXPR: w_modify_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gINIT_EXPR: w_init_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTARGET_EXPR: w_target_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOND_EXPR: w_cond_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIND_EXPR: w_bind_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCALL_EXPR: w_call_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMETHOD_CALL_EXPR: w_method_call_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gWITH_CLEANUP_EXPR: w_with_cleanup_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCLEANUP_POINT_EXPR: w_cleanup_point_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPLACEHOLDER_EXPR: w_placeholder_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gWITH_RECORD_EXPR: w_with_record_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPLUS_EXPR: w_plus_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMINUS_EXPR: w_minus_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMULT_EXPR: w_mult_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUNC_DIV_EXPR: w_trunc_div_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCEIL_DIV_EXPR: w_ceil_div_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFLOOR_DIV_EXPR: w_floor_div_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gROUND_DIV_EXPR: w_round_div_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUNC_MOD_EXPR: w_trunc_mod_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCEIL_MOD_EXPR: w_ceil_mod_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFLOOR_MOD_EXPR: w_floor_mod_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gROUND_MOD_EXPR: w_round_mod_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRDIV_EXPR: w_rdiv_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXACT_DIV_EXPR: w_exact_div_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFIX_TRUNC_EXPR: w_fix_trunc_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFIX_CEIL_EXPR: w_fix_ceil_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFIX_FLOOR_EXPR: w_fix_floor_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFIX_ROUND_EXPR: w_fix_round_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFLOAT_EXPR: w_float_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXPON_EXPR: w_expon_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNEGATE_EXPR: w_negate_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMIN_EXPR: w_min_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMAX_EXPR: w_max_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gABS_EXPR: w_abs_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFFS_EXPR: w_ffs_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLSHIFT_EXPR: w_lshift_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRSHIFT_EXPR: w_rshift_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLROTATE_EXPR: w_lrotate_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRROTATE_EXPR: w_rrotate_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_IOR_EXPR: w_bit_ior_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_XOR_EXPR: w_bit_xor_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_AND_EXPR: w_bit_and_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_ANDTC_EXPR: w_bit_andtc_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBIT_NOT_EXPR: w_bit_not_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_ANDIF_EXPR: w_truth_andif_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_ORIF_EXPR: w_truth_orif_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_AND_EXPR: w_truth_and_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_OR_EXPR: w_truth_or_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_XOR_EXPR: w_truth_xor_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRUTH_NOT_EXPR: w_truth_not_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLT_EXPR: w_lt_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLE_EXPR: w_le_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gGT_EXPR: w_gt_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gGE_EXPR: w_ge_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEQ_EXPR: w_eq_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNE_EXPR: w_ne_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNORDERED_EXPR: w_unordered_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gORDERED_EXPR: w_ordered_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNLT_EXPR: w_unlt_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNLE_EXPR: w_unle_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNGT_EXPR: w_ungt_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNGE_EXPR: w_unge_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNEQ_EXPR: w_uneq_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gIN_EXPR: w_in_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSET_LE_EXPR: w_set_le_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCARD_EXPR: w_card_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRANGE_EXPR: w_range_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCONVERT_EXPR: w_convert_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNOP_EXPR: w_nop_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNON_LVALUE_EXPR: w_non_lvalue_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSAVE_EXPR: w_save_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUNSAVE_EXPR: w_unsave_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRTL_EXPR: w_rtl_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gADDR_EXPR: w_addr_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREFERENCE_EXPR: w_reference_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gENTRY_VALUE_EXPR: w_entry_value_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPLEX_EXPR: w_complex_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCONJ_EXPR: w_conj_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREALPART_EXPR: w_realpart_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gIMAGPART_EXPR: w_imagpart_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPREDECREMENT_EXPR: w_predecrement_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPREINCREMENT_EXPR: w_preincrement_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPOSTDECREMENT_EXPR: w_postdecrement_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPOSTINCREMENT_EXPR: w_postincrement_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVA_ARG_EXPR: w_va_arg_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRY_CATCH_EXPR: w_try_catch_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRY_FINALLY_EXPR: w_try_finally_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gGOTO_SUBROUTINE_EXPR: w_goto_subroutine_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPOPDHC_EXPR: w_popdhc_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPOPDCC_EXPR: w_popdcc_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLABEL_EXPR: w_label_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gGOTO_EXPR: w_goto_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRETURN_EXPR: w_return_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXIT_EXPR: w_exit_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLOOP_EXPR: w_loop_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLABELED_BLOCK_EXPR: w_labeled_block_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXIT_BLOCK_EXPR: w_exit_block_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXPR_WITH_FILE_LOCATION: w_expr_with_file_location (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSWITCH_EXPR: w_switch_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gSRCLOC: w_srcloc (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSIZEOF_EXPR: w_sizeof_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gARROW_EXPR: w_arrow_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gALIGNOF_EXPR: w_alignof_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEXPR_STMT: w_expr_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCOMPOUND_STMT: w_compound_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gDECL_STMT: w_decl_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gIF_STMT: w_if_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFOR_STMT: w_for_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gWHILE_STMT: w_while_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gDO_STMT: w_do_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRETURN_STMT: w_return_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBREAK_STMT: w_break_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCONTINUE_STMT: w_continue_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSWITCH_STMT: w_switch_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gGOTO_STMT: w_goto_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLABEL_STMT: w_label_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gASM_STMT: w_asm_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSCOPE_STMT: w_scope_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCASE_LABEL: w_case_label (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSTMT_EXPR: w_stmt_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gOFFSET_REF: w_offset_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPTRMEM_CST: w_ptrmem_cst (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gNEW_EXPR: w_new_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVEC_NEW_EXPR: w_vec_new_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gDELETE_EXPR: w_delete_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gVEC_DELETE_EXPR: w_vec_delete_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSCOPE_REF: w_scope_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gMEMBER_REF: w_member_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTYPE_EXPR: w_type_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gAGGR_INIT_EXPR: w_aggr_init_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTHROW_EXPR: w_throw_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gEMPTY_CLASS_EXPR: w_empty_class_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTEMPLATE_DECL: w_template_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTEMPLATE_PARM_INDEX: w_template_parm_index (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTEMPLATE_TYPE_PARM: w_template_type_parm (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTEMPLATE_TEMPLATE_PARM: w_template_template_parm (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBOUND_TEMPLATE_TEMPLATE_PARM: w_bound_template_template_parm (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTYPENAME_TYPE: w_typename_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTYPEOF_TYPE: w_typeof_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUSING_DECL: w_using_decl (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gDEFAULT_ARG: w_default_arg (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTEMPLATE_ID_EXPR: w_template_id_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCPLUS_BINDING: w_cplus_binding (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gOVERLOAD: w_overload (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gWRAPPER: w_wrapper (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLOOKUP_EXPR: w_lookup_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gFUNCTION_NAME: w_function_name (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gMODOP_EXPR: w_modop_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gCAST_EXPR: w_cast_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gREINTERPRET_CAST_EXPR: w_reinterpret_cast_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gCONST_CAST_EXPR: w_const_cast_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gSTATIC_CAST_EXPR: w_static_cast_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gDYNAMIC_CAST_EXPR: w_dynamic_cast_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gDOTSTAR_EXPR: w_dotstar_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gTYPEID_EXPR: w_typeid_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case gPSEUDO_DTOR_EXPR: w_pseudo_dtor_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSUBOBJECT: w_subobject (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCTOR_STMT: w_ctor_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCLEANUP_STMT: w_cleanup_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSTART_CATCH_STMT: w_start_catch_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gCTOR_INITIALIZER: w_ctor_initializer (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRETURN_INIT: w_return_init (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTRY_BLOCK: w_try_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gHANDLER: w_handler (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gTAG_DEFN: w_tag_defn (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gIDENTITY_CONV: w_identity_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gLVALUE_CONV: w_lvalue_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gQUAL_CONV: w_qual_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gSTD_CONV: w_std_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPTR_CONV: w_ptr_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gPMEM_CONV: w_pmem_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gBASE_CONV: w_base_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gREF_BIND: w_ref_bind (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gUSER_CONV: w_user_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gAMBIG_CONV: w_ambig_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case gRVALUE_CONV: w_rvalue_conv (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ASG_ND: w_dc_asg_nd (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_IDENTIFIER: w_dc_identifier (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_TYPE: w_dc_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_BUILT_IN_TYPE: w_dc_built_in_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_AGGR_TYPE: w_dc_aggr_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ENUM_TYPE: w_dc_enum_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_TEMPL_PARM_TYPE: w_dc_templ_parm_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_GENERIC_TYPE: w_dc_generic_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_GENERATED_TYPE: w_dc_generated_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ALIAS_TYPE: w_dc_alias_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FORWARD_TYPE: w_dc_forward_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARRAY_TYPE: w_dc_array_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_PTR_TYPE: w_dc_ptr_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_REF_TYPE: w_dc_ref_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FCT_TYPE: w_dc_fct_type (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SCOPE: w_dc_scope (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SYSTEM: w_dc_system (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SCOPE_GLB: w_dc_scope_glb (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SCOPE_COMPIL: w_dc_scope_compile (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SCOPE_FILE: w_dc_scope_file (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_PACKAGE: w_dc_package (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_NAME_SPACE: w_dc_name_space (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_BLOCK: w_dc_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_STATIC_BLOCK: w_dc_static_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_TRY_BLOCK: w_dc_try_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_CATCH_BLOCK: w_dc_catch_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FINALLY_BLOCK: w_dc_finally_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SYNCHRONIZED_BLOCK: w_dc_synchronized_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_OBJECT: w_dc_object (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FORMAL_FCT_PARAM: w_dc_formal_fct_param (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FORMAL_CATCH_PARAM: w_dc_formal_catch_param (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FUNCTION: w_dc_function (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_LABEL: w_dc_label (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_CASE_LABEL: w_dc_case_label (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_DEFAULT_LABEL: w_dc_default_label (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ENUMERATOR: w_dc_enumerator (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_EXPRESSION: w_dc_expression (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_EMPTY_EXPR: w_dc_empty_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_LITERAL: w_dc_literal (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SYNTAX_ERROR: w_dc_syntax_error (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_NAME_REF: w_dc_name_ref (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_OPERATOR: w_dc_operator (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_UNARY_OPERATOR: w_dc_unary_operator (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_BINARY_OPERATOR: w_dc_binary_operator (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_QUESTION_OPERATOR: w_dc_question_operator (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_INIT_LIST: w_dc_init_list (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ACTUAL_FCT_PARAM_LIST: w_dc_actual_fct_param_list (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ACTUAL_TEMPL_PARAM_LIST: w_dc_actual_templ_param_list (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_CREATION_EXPR: w_dc_creation_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_DESTROY_EXPR: w_dc_destroy_expr (g, i, & g-> nodeBuffer [i], f, 0); break;
//	case dC_CNTRL_STMT: w_dc_cntrl_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SELECTION: w_dc_selection (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_IF: w_dc_if (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SWITCH: w_dc_switch (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_JUMP: w_dc_jump (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_GOTO: w_dc_goto (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_RETURN: w_dc_return (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_BREAK: w_dc_break (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_CONTINUE: w_dc_continue (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_LOOP: w_dc_loop (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_DO_WHILE_LOOP: w_dc_do_while_loop (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_WHILE_DO_LOOP: w_dc_while_do_loop (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_FOR_LOOP: w_dc_for_loop (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_SQL_STMT: w_dc_sql_stmt (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_COMMENT: w_dc_comment (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_JAVA_DOC_COMMENT: w_dc_java_doc_comment (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_THROW_SPEC: w_dc_throw_spec (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ASM_BLOCK: w_dc_asm_block (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC: w_dc_arc (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_REFERENCE: w_dc_arc_reference (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_INSTANCE: w_dc_instance (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_INHERITANCE: w_dc_inheritance (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_IMPLEMENTS: w_dc_implements (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_FRIEND: w_dc_arc_friend (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_DEF_LOC: w_dc_arc_def_loc (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_REFERS_TO: w_dc_refers_to (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_SYNTAX: w_dc_arc_syntax (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_SON: w_dc_arc_son (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_TRUE: w_dc_arc_true (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_FALSE: w_dc_arc_false (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_OPD: w_dc_arc_opd (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_IMPORT: w_dc_import (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_COND: w_dc_arc_cond (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_FOR_INIT: w_dc_arc_for_init (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_FOR_INCR: w_dc_arc_for_incr (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_INIT_VAL: w_dc_arc_init_val (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_BIT_DIM: w_dc_arc_bit_dim (g, i, & g-> nodeBuffer [i], f, 0); break;
	case dC_ARC_ARRAY_DIM: w_dc_arc_array_dim (g, i, & g-> nodeBuffer [i], f, 0); break;
	}
}

int w_gd_graph_common (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    f (g, np, & p-> common. other_node, n++);
    return n;
}

int w_gcc_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> decl. name, n++);
    f (g, np, & p-> decl. mngl, n++);
    f (g, np, & p-> decl. type, n++);
    f (g, np, & p-> decl. chan, n++);
    f (g, np, & p-> decl. scpe, n++);
    return n;
}

int w_namespace_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> namespace_decl. dcls, n++);
    f (g, np, & p-> namespace_decl. alis, n++);
    return n;
}
 
int w_function_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> function_decl. args, n++);
    f (g, np, & p-> function_decl. body, n++);
    return n;
}

int w_var_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> var_decl. init, n++);
    f (g, np, & p-> var_decl. size, n++);
    return n;
}

int w_parm_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> parm_decl. argt, n++);
    f (g, np, & p-> parm_decl. size, n++);
    return n;
}

int w_type_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    return n;
}

int w_label_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    return n;
}

int w_const_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> const_decl. cnst, n++);
    return n;
}

int w_result_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> result_decl. init, n++);
    f (g, np, & p-> result_decl. size, n++);
    return n;
}

int w_field_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> field_decl. init, n++);
    f (g, np, & p-> field_decl. size, n++);
    f (g, np, & p-> field_decl. bpos, n++);
    return n;
}

int w_template_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    f (g, np, & p-> template_decl. rslt, n++);
    f (g, np, & p-> template_decl. inst, n++);
    f (g, np, & p-> template_decl. spcs, n++);
    f (g, np, & p-> template_decl. parms, n++);
    return n;
}

int w_using_decl (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_decl (g, np, p, f, n);
    return n;
}


int w_gcc_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> type. name, n++);
    f (g, np, & p-> type. size, n++);
    f (g, np, & p-> type. unql, n++);
   
    return n;
}

int w_function_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> function_type. retn, n++);
    f (g, np, & p-> function_type. prms, n++);
    return n;
}

int w_integer_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> integer_type. min, n++);
    f (g, np, & p-> integer_type. max, n++);
    return n;
}

int w_array_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> array_type. elts, n++);
    f (g, np, & p-> array_type. domn, n++);
    return n;
}

int w_void_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_real_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_complex_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_vector_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_enumeral_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> enumeral_type. min, n++);
    f (g, np, & p-> enumeral_type. max, n++);
    f (g, np, & p-> enumeral_type. csts, n++);
    return n;
}

int w_boolean_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_char_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_pointer_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> pointer_type. ptd, n++);
    f (g, np, & p-> pointer_type. cls, n++);
    return n;
}

int w_offset_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_reference_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> reference_type. refd, n++);
    return n;
}

int w_method_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> method_type. clas, n++);
    f (g, np, & p-> method_type. retn, n++);
    f (g, np, & p-> method_type. prms, n++);
    return n;
}

int w_file_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_set_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_record_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> record_type. flds, n++);
    f (g, np, & p-> record_type. fncs, n++);
    f (g, np, & p-> record_type. ptd, n++);
    f (g, np, & p-> record_type. cls, n++);
    f (g, np, & p-> record_type. vfld, n++);
    f (g, np, & p-> record_type. binf, n++);
    return n;
}

int w_union_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    f (g, np, & p-> union_type. flds, n++);
    f (g, np, & p-> union_type. fncs, n++);
    f (g, np, & p-> union_type. ptd, n++);  
    f (g, np, & p-> union_type. cls, n++);  
    f (g, np, & p-> union_type. vfld, n++);
    f (g, np, & p-> union_type. binf, n++);
    return n;
}

int w_qual_union_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_lang_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_template_type_parm (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

 
int w_template_parm_index (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_template_template_parm (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_bound_template_template_parm (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_typename_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_typeof_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_type (g, np, p, f, n);
    return n;
}

int w_gcc_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> expr. type, n++);
    return n;
}

int w_gcc_expr1 (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> expr1. op0, n++);
    return n;
}

int w_gcc_expr2 (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> expr2. op0, n++);
    f (g, np, & p-> expr2. op1, n++);
    return n;
}

int w_gcc_expr_ineq (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> expr_ineq. op0, n++);
    f (g, np, & p-> expr_ineq. op1, n++);
    return n;
}
 
int w_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_compound_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> compound_stmt. body, n++);
    f (g, np, & p-> compound_stmt. next, n++);
    return n;
}

int w_expr_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> expr_stmt. texpr, n++);
    f (g, np, & p-> expr_stmt. next, n++);
    return n;
}

int w_decl_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> decl_stmt. decl, n++);
    f (g, np, & p-> decl_stmt. next, n++);
    return n;
}

int w_scope_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> scope_stmt. next, n++);
    return n;
}

int w_return_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> return_stmt. ret_expr, n++);
    f (g, np, & p-> return_stmt. next, n++);
    return n;
}

int w_stmt_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> stmt_expr. stmt, n++);
    return n;
}

int w_break_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> break_stmt. next, n++);
    return n;
}

int w_continue_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> continue_stmt. next, n++);
    return n;
}

int w_if_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> if_stmt. cond, n++);
    f (g, np, & p-> if_stmt. thenc, n++);
    f (g, np, & p-> if_stmt. elsec, n++);
    f (g, np, & p-> if_stmt. next, n++);
    return n;
}

int w_switch_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> switch_stmt. cond, n++);
    f (g, np, & p-> switch_stmt. body, n++);
    f (g, np, & p-> switch_stmt. next, n++);
    return n;
}

int w_label_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> label_stmt. label, n++);
    f (g, np, & p-> label_stmt. next, n++);
    return n;
}

int w_try_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{  
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> try_block. body, n++);
    f (g, np, & p-> try_block. hdlr, n++);
    f (g, np, & p-> try_block. next, n++);
    return n;
}

int w_handler (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{  
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> handler. body, n++);
    f (g, np, & p-> handler. next, n++);
    return n;
}

int w_start_catch_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{  
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> start_catch_stmt. catch_type, n++);  
    f (g, np, & p-> start_catch_stmt. next, n++);
    return n;
}

int w_cleanup_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{  
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> cleanup_stmt. decl, n++);
    f (g, np, & p-> cleanup_stmt. clean_expr, n++);
    f (g, np, & p-> cleanup_stmt. next, n++);
    return n;
}

int w_case_label (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> case_label. low, n++);
    f (g, np, & p-> case_label. high, n++);
    f (g, np, & p-> case_label. next, n++);
    return n;
}

int w_goto_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> goto_stmt. dest, n++);
    f (g, np, & p-> goto_stmt. next, n++);
    return n;
}

int w_for_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> for_stmt. init, n++);
    f (g, np, & p-> for_stmt. cond, n++);
    f (g, np, & p-> for_stmt. inc_expr, n++);
    f (g, np, & p-> for_stmt. body, n++);
    f (g, np, & p-> for_stmt. next, n++);
    return n;
}

int w_while_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> while_stmt. body, n++);
    f (g, np, & p-> while_stmt. cond, n++);
    f (g, np, & p-> while_stmt. next, n++);
    return n;
}

int w_do_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> do_stmt. body, n++);
    f (g, np, & p-> do_stmt. cond, n++);
    f (g, np, & p-> do_stmt. next, n++);
    return n;
}

int w_asm_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> asm_stmt. strg, n++);
    f (g, np, & p-> asm_stmt. outs, n++);
    f (g, np, & p-> asm_stmt. ins, n++);
    f (g, np, & p-> asm_stmt. clbr, n++);
    f (g, np, & p-> asm_stmt. next, n++);
    return n;
}


 
int w_component_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> component_ref. op0, n++);
    f (g, np, & p-> component_ref. op1, n++);
    return n;
}

int w_bit_field_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_indirect_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> indirect_ref. op0, n++);
    return n;
}

int w_buffer_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_array_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> array_ref. op0, n++);
    f (g, np, & p-> array_ref. op1, n++);
    return n;
}

int w_offset_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> offset_ref. op0, n++);
    f (g, np, & p-> offset_ref. op1, n++);
    return n;
}

int w_scope_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> scope_ref. op0, n++);
    f (g, np, & p-> scope_ref. op1, n++);
    return n;
}

int w_member_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> member_ref. op0, n++);
    f (g, np, & p-> member_ref. op1, n++);
    return n;
}

int w_constructor (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> constructor. elts, n++);
    return n;
}
 
int w_compound_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> compound_expr. op0, n++);
    f (g, np, & p-> compound_expr. op1, n++);
    return n;
}

int w_modify_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> modify_expr. op0, n++);
    f (g, np, & p-> modify_expr. op1, n++);
    return n;
}

int w_init_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> init_expr. op0, n++);
    f (g, np, & p-> init_expr. op1, n++);
    return n;
}

int w_throw_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> throw_expr. op0, n++);
    return n;
}

int w_target_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> target_expr. decl, n++);
    f (g, np, & p-> target_expr. init, n++);
    f (g, np, & p-> target_expr. clnp, n++);
    return n;
}

int w_cond_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> cond_expr. op0, n++);
    f (g, np, & p-> cond_expr. op1, n++);
    f (g, np, & p-> cond_expr. op2, n++);
    return n;
}

int w_bind_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> bind_expr. vars, n++);
    f (g, np, & p-> bind_expr. body, n++);
    return n;
}

int w_call_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> call_expr. fn, n++);
    f (g, np, & p-> call_expr. args, n++);
    return n;
}

int w_method_call_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_with_cleanup_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_cleanup_point_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> cleanup_point_expr. op0, n++);
    return n;
}

int w_placeholder_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{  
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_with_record_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_plus_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_minus_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_mult_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_trunc_div_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_ceil_div_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_floor_div_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_round_div_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_trunc_mod_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_ceil_mod_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_floor_mod_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_round_mod_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_rdiv_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_exact_div_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_fix_trunc_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_fix_ceil_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_fix_floor_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_fix_round_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_float_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_expon_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_negate_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_min_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_max_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_abs_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_ffs_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_lshift_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_rshift_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_lrotate_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_rrotate_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_bit_ior_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_bit_xor_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_bit_and_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_bit_andtc_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_bit_not_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_truth_andif_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_andif_expr. op0, n++);
    f (g, np, & p-> truth_andif_expr. op1, n++);
    return n;
}

int w_truth_orif_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_orif_expr. op0, n++);
    f (g, np, & p-> truth_orif_expr. op1, n++);
    return n;
}

int w_truth_and_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_and_expr. op0, n++);
    f (g, np, & p-> truth_and_expr. op1, n++);
    return n;
}

int w_truth_or_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_or_expr. op0, n++);
    f (g, np, & p-> truth_or_expr. op1, n++);
    return n;
}

int w_truth_xor_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_xor_expr. op0, n++);
    f (g, np, & p-> truth_xor_expr. op1, n++);
    return n;
}

int w_truth_not_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> truth_not_expr. op0, n++);
    return n;
}

int w_lt_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_le_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_gt_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_ge_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_eq_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_ne_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_unordered_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_ordered_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_unlt_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_unle_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_ungt_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_unge_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_uneq_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_in_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_set_le_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr_ineq (g, np, p, f, n);
    return n;
}

int w_card_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_range_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_convert_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_nop_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_non_lvalue_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_save_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> save_expr. op0, n++);
    return n;
}

int w_unsave_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_rtl_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_addr_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> addr_expr. op0, n++);
    return n;
}

int w_reference_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_entry_value_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_complex_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr2 (g, np, p, f, n);
    return n;
}

int w_conj_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_realpart_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr1 (g, np, p, f, n);
    return n;
}

int w_predecrement_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> predecrement_expr. op0, n++);
    f (g, np, & p-> predecrement_expr. op1, n++);
    return n;
}

int w_preincrement_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> preincrement_expr. op0, n++);
    f (g, np, & p-> preincrement_expr. op1, n++);
    return n;
}

int w_postdecrement_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> postdecrement_expr. op0, n++);
    f (g, np, & p-> postdecrement_expr. op1, n++);
    return n;
}

int w_postincrement_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> postincrement_expr. op0, n++);
    f (g, np, & p-> postincrement_expr. op1, n++);
    return n;
}

int w_va_arg_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> va_arg_expr. op0, n++);
    return n;
}

int w_try_catch_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> try_catch_expr. op0, n++);
    f (g, np, & p-> try_catch_expr. op1, n++);
    return n;
}

int w_try_finally_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> try_finally_expr. op0, n++);
    f (g, np, & p-> try_finally_expr. op1, n++);
    return n;
}

int w_goto_subroutine_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> goto_subroutine_expr. op0, n++);
    f (g, np, & p-> goto_subroutine_expr. op1, n++);
    return n;
}
 
int w_popdhc_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_popdcc_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_label_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_goto_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_return_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_exit_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> exit_expr. cond, n++);
    return n;
}

int w_loop_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> loop_expr. body, n++);
    return n;
}
 
int w_labeled_block_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_exit_block_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_expr_with_file_location (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> expr_with_file_location. expr2, n++);
    return n;
}

int w_switch_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> switch_expr. op0, n++);
    f (g, np, & p-> switch_expr. op1, n++);
    return n;
}

int w_sizeof_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> sizeof_expr. op0, n++);
    return n;
}

int w_arrow_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> arrow_expr. op0, n++);
    return n;
}
  
int w_alignof_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> alignof_expr. op0, n++);
    return n;
}
  
int w_new_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> new_expr. op0, n++);
    return n;
}

int w_vec_new_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> vec_new_expr. op0, n++);
    return n;
}

int w_delete_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> delete_expr. op0, n++);
    return n;
}

int w_vec_delete_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> vec_delete_expr. op0, n++);
    return n;
}

int w_type_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> type_expr. op0, n++);
    return n;
}

int w_aggr_init_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> aggr_init_expr. fn, n++);
    f (g, np, & p-> aggr_init_expr. args, n++);
    f (g, np, & p-> aggr_init_expr. decl, n++);
    return n;
}

int w_empty_class_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_template_id_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> template_id_expr. op0, n++);
    f (g, np, & p-> template_id_expr. op1, n++);
    return n;
}

int w_lookup_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> lookup_expr. op0, n++);
    return n;
}

int w_function_name (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_subobject (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> subobject. clnp, n++);
    f (g, np, & p-> subobject. next, n++);
    return n;
}

int w_ctor_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> ctor_stmt. next, n++);
    return n;
}

int w_ctor_initializer (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> ctor_initializer. op0, n++);
    f (g, np, & p-> ctor_initializer. op1, n++);
    return n;
}

int w_return_init (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> return_init. op0, n++);
    f (g, np, & p-> return_init. op1, n++);
    return n;
}

int w_tag_defn (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    return n;
}

int w_identity_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> identity_conv. op0, n++);
    return n;
}

int w_lvalue_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> lvalue_conv. op0, n++);
    return n;
}

int w_qual_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> qual_conv. op0, n++);
    return n;
}

int w_std_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> std_conv. op0, n++);
    return n;
}

int w_ptr_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> ptr_conv. op0, n++);
    return n;
}

int w_pmem_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> pmem_conv. op0, n++);
    return n;
}

int w_base_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> base_conv. op0, n++);
    return n;
}

int w_ref_bind (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> ref_bind. op0, n++);
    return n;
}

int w_user_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> user_conv. op0, n++);
    f (g, np, & p-> user_conv. op1, n++);
    return n;
}

int w_ambig_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> ambig_conv. op0, n++);
    return n;
}

int w_rvalue_conv (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_expr (g, np, p, f, n);
    f (g, np, & p-> rvalue_conv. op0, n++);
    return n;
}

int w_op_identifier (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_identifier_node (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_gcc_const (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> gcc_cst. type, n++);
    return n;
}

int w_integer_cst (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_const (g, np, p, f, n);
    return n;
}

int w_real_cst (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_const (g, np, p, f, n);
    return n;
}

int w_ptrmem_cst (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_const (g, np, p, f, n);
    f (g, np, & p-> ptrmem_cst. clas, n++);
    f (g, np, & p-> ptrmem_cst. mbr, n++);
    return n;
}

int w_complex_cst (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_const (g, np, p, f, n);
    return n;
}

int w_string_cst (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gcc_const (g, np, p, f, n);
    return n;
}

int w_tree_list (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> tree_list. purpose, n++);
    f (g, np, & p-> tree_list. value, n++);
    f (g, np, & p-> tree_list. chan, n++);
    return n;
}

int w_tree_vec (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> tree_vec. inherit, n++);
    f (g, np, & p-> tree_vec. type, n++);
    f (g, np, & p-> tree_vec. base, n++);
    return n;
}

int w_error_mark (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_default_arg (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_cplus_binding (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_overload (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> overload. crnt, n++);
    f (g, np, & p-> overload. chan, n++);
    return n;
}

int w_wrapper (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_dc_asg_nd (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    return n;
}

int w_dc_identifier (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n) 
{
    n = w_dc_asg_nd (g, np, p, f, n);
    f (g, np, & p-> dc_identifier. instance, n++);   
    f (g, np, & p-> dc_identifier. defLoc, n++);     
    f (g, np, & p-> dc_identifier. arcFriend, n++);  
    f (g, np, & p-> dc_identifier. array_dim, n++);  
    return n;
}

int w_dc_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);
    f (g, np, & p-> dc_type. implements, n++);  
    return n;
}

int w_dc_built_in_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_aggr_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_aggr_type. members, n++);  
    f (g, np, & p-> dc_aggr_type. inheritance, n++);  
    return n;
}

int w_dc_enum_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_enum_type. values, n++);  
    return n;
}

int w_dc_templ_parm_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_generic_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_generic_type. data_functs, n++);  
    f (g, np, & p-> dc_generic_type. formals, n++);      
    return n;
}

int w_dc_generated_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_generated_type. actuals, n++);      
    return n;
}

int w_dc_alias_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_alias_type. original, n++);     
    return n;
}

int w_dc_forward_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_array_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_ptr_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_ref_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    return n;
}

int w_dc_fct_type (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_type (g, np, p, f, n);
    f (g, np, & p-> dc_fct_type. formalParams, n++);  
    return n;
}

int w_dc_scope (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);
    f (g, np, & p-> dc_scope. contents, n++);  
    return n;
}

int w_dc_system (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_scope_glb (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_scope_compile (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_scope_file (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_package (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_name_space (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_static_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_try_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_catch_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_finally_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_synchronized_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_scope (g, np, p, f, n);
    return n;
}

int w_dc_object (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);
    f (g, np, & p-> dc_object. bitfield_dim, n++);  
    f (g, np, & p-> dc_object. initial_val, n++);  
    return n;
}

int w_dc_formal_fct_param (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_object (g, np, p, f, n);
    return n;
}

int w_dc_formal_catch_param (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_object (g, np, p, f, n);
    return n;
}

int w_dc_function (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);

    f (g, np, & p-> dc_function. formal_fct, n++);  
    f (g, np, & p-> dc_function. exception, n++);  
    f (g, np, & p-> dc_function. block, n++);   
    return n;
}

int w_dc_label (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);
    return n;
}

int w_dc_case_label (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_label (g, np, p, f, n);
    f (g, np, & p-> dc_case_label. value, n++);   
    return n;
}

int w_dc_default_label (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_label (g, np, p, f, n);
    return n;
}

int w_dc_enumerator (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_identifier (g, np, p, f, n);
    f (g, np, & p-> dc_enumerator. value, n++);  
    return n;
}

int w_dc_expression (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    f (g, np, & p-> dc_expression. refers, n++);  
    return n;
}

int w_dc_empty_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    return n;
}

int w_dc_literal (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_literal. type, n++);  
    return n;
}

int w_dc_syntax_error (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    return n;
}

int w_dc_name_ref (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    return n;
}

int w_dc_operator (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_operator. ops, n++);  
    return n;
}

int w_dc_unary_operator (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_operator (g, np, p, f, n);
    return n;
}

int w_dc_binary_operator (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_operator (g, np, p, f, n);
    return n;
}

int w_dc_question_operator (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_operator (g, np, p, f, n);
    return n;
}

int w_dc_init_list (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_init_list. init, n++);  
    return n;
}

int w_dc_actual_fct_param_list (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_actual_fct_param_list. actuals, n++);  
    return n;
}

int w_dc_actual_templ_param_list (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_actual_templ_param_list. actuals, n++);  
    return n;
}

int w_dc_creation_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_creation_expr. init, n++);  
    return n;
}

int w_dc_destroy_expr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_expression (g, np, p, f, n);
    f (g, np, & p-> dc_destroy_expr. what, n++);  
    f (g, np, & p-> dc_destroy_expr. array, n++);  
    return n;
}

int w_dc_ctrl_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    return n;
}

int w_dc_selection (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_ctrl_stmt (g, np, p, f, n);
    return n;
}

int w_dc_if (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_ctrl_stmt (g, np, p, f, n);
    f (g, np, & p-> dc_if. cond, n++);      
    f (g, np, & p-> dc_if. true_block, n++);  
    f (g, np, & p-> dc_if. false_block, n++); 
    return n;
}

int w_dc_switch (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_ctrl_stmt (g, np, p, f, n);
    f (g, np, & p-> dc_switch. cond, n++);  
    f (g, np, & p-> dc_switch. body, n++);  
    return n;
}

int w_dc_jump (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_ctrl_stmt (g, np, p, f, n);
    return n;
}

int w_dc_goto (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_jump (g, np, p, f, n);
    f (g, np, & p-> dc_goto. dest, n++);  
    return n;
}

int w_dc_return (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_jump (g, np, p, f, n);
    f (g, np, & p-> dc_return. value, n++);  
    return n;
}

int w_dc_break (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_jump (g, np, p, f, n);
    f (g, np, & p-> dc_break. dest, n++);  
    return n;
}

int w_dc_continue (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_jump (g, np, p, f, n);
    f (g, np, & p-> dc_continue. dest, n++);  
    return n;
}

int w_dc_loop (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_ctrl_stmt (g, np, p, f, n);
    return n;
}

int w_dc_do_while_loop (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_loop (g, np, p, f, n);
    f (g, np, & p-> dc_do_while_loop. cond, n++);  
    f (g, np, & p-> dc_do_while_loop. block, n++);  
    return n;
}

int w_dc_while_do_loop (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_loop (g, np, p, f, n);
    f (g, np, & p-> dc_while_do_loop. cond, n++);  
    f (g, np, & p-> dc_while_do_loop. block, n++);  
    return n;
}

int w_dc_for_loop (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_loop (g, np, p, f, n);
    f (g, np, & p-> dc_for_loop. init, n++);  
    f (g, np, & p-> dc_for_loop. cond, n++);  
    f (g, np, & p-> dc_for_loop. incr, n++);  
    f (g, np, & p-> dc_for_loop. block, n++);  
    return n;
}

int w_dc_sql_stmt (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    return n;
}

int w_dc_comment (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    return n;
}

int w_dc_java_doc_comment (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_comment (g, np, p, f, n);
    return n;
}

int w_dc_throw_spec (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    f (g, np, & p-> dc_throw_spec. except, n++);   
    return n;
}

int w_dc_asm_block (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_asg_nd (g, np, p, f, n);
    return n;
}

int w_dc_arc (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_gd_graph_common (g, np, p, f, n);
    f (g, np, & p-> dc_arc. src, n++);
    f (g, np, & p-> dc_arc. dest, n++);
    f (g, np, & p-> dc_arc. next, n++);   
    return n;
}

int w_dc_arc_reference (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc (g, np, p, f, n);
    return n;
}

int w_dc_instance (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_inheritance (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_implements (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_arc_friend (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_arc_def_loc (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_refers_to (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_reference (g, np, p, f, n);
    return n;
}

int w_dc_arc_syntax (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc (g, np, p, f, n);
    return n;
}

int w_dc_arc_son (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_true (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_false (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_opd (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_import (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_cond (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_for_init (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_for_incr (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_init_val (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_bit_dim (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

int w_dc_arc_array_dim (gd_graph *g, node_ptr np, gd_graph_node *p, void *f (gd_graph *, node_ptr, node_ptr *, int), int n)
{
    n = w_dc_arc_syntax (g, np, p, f, n);
    return n;
}

