/*+
 * Name:	gd_while.c
 *
 * Purpose:	Transforms gcc while_stmt and do_stmt to datrix do_while and
 * 		while_do.
 *              -- Uses other_ptr to maintain the link between statements
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 26, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void do_while_do(gd_graph *g, node_ptr i);
void do_do_while(gd_graph *g, node_ptr i);
node_ptr nextNode(gd_graph * g, node_ptr i);
void setNext(gd_graph * g, node_ptr i, node_ptr val);

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  for(i = 0; i < g.curNodePos;i++){
    if (g.nodeBuffer[i].common.type == gWHILE_STMT){
      do_while_do(&g,i);
    }
    if (g.nodeBuffer[i].common.type == gDO_STMT){
      do_do_while(&g,i);
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}

void do_while_do(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from break_stmt */
  int line;		/* src line				*/
  node_ptr next;	/* next in chain of statments		*/
  node_ptr body;	/* body of while loop			*/
  node_ptr cond;	/* condition of while loop		*/

  /* other locals */
  node_ptr new_node,
	   prev_node,
	   tmp;

  /* stuff to save from the current node */
  line = g->nodeBuffer[i].while_stmt.line;
  next = g->nodeBuffer[i].while_stmt.next;
  body = g->nodeBuffer[i].while_stmt.body;
  cond = g->nodeBuffer[i].while_stmt.cond;
  if (g->nodeBuffer[cond].common.type == gTREE_LIST){
    /* has a scope node */
    cond = g-> nodeBuffer[cond].tree_list.value;
  }

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_WHILE_DO_LOOP;

  g->nodeBuffer[i].dc_asg_nd.file = -1;
  g->nodeBuffer[i].dc_asg_nd.line = line;

  /* store chain in other_node, since datrix nodes do not have <next> ptrs */
  g->nodeBuffer[i].common.other_node = next;

  /* arc cond to condition node */
  new_node = newNode(g);
  if (new_node == -1){
    fprintf(stderr,"error allocating node\n");
    exit(1);
  }
  g->nodeBuffer[new_node].common.other_node = -1;
  g->nodeBuffer[new_node].common.type = dC_ARC_COND;
  g->nodeBuffer[new_node].dc_arc.src = i;
  g->nodeBuffer[new_node].dc_arc.dest = cond;
  g->nodeBuffer[new_node].dc_arc.ordinal = 0;
  g->nodeBuffer[new_node].dc_arc.next = -1;
  g-> nodeBuffer[i].dc_while_do_loop.cond = new_node;

  g-> nodeBuffer[i].dc_while_do_loop.block = -1;
  prev_node = -1;
  while(body != -1){
    if (g->nodeBuffer[body].common.type == gSCOPE_STMT) {
      body = g->nodeBuffer[body].scope_stmt.next;
    } else {
      new_node = newNode(g);
      if (new_node == -1){
	fprintf(stderr,"error allocating node\n");
	exit(1);
      }
      g->nodeBuffer[new_node].common.other_node = -1;
      g->nodeBuffer[new_node].common.type = dC_ARC_SON;
      g->nodeBuffer[new_node].dc_arc.src = i;
      g->nodeBuffer[new_node].dc_arc.dest = body;
      g->nodeBuffer[new_node].dc_arc.ordinal = 0;
      g->nodeBuffer[new_node].dc_arc.next = -1;
      if (prev_node != -1){
	g->nodeBuffer[prev_node].dc_arc.next = new_node;
      }
      prev_node = new_node;
      if (g->nodeBuffer[i].dc_while_do_loop.block == -1){
	g->nodeBuffer[i].dc_while_do_loop.block = new_node;
      }
      tmp = body;
      body = nextNode(g,body);
      /* erase the chain*/
      setNext(g,tmp,-1);
    }
  }
}

void do_do_while(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from break_stmt */
  int line;		/* src line				*/
  node_ptr next;	/* next in chain of statments		*/
  node_ptr body;	/* body of while loop			*/
  node_ptr cond;	/* condition of while loop		*/

  /* other locals */
  node_ptr new_node,
	   prev_node,
	   tmp;

  /* stuff to save from the current node */
  line = g->nodeBuffer[i].do_stmt.line;
  next = g->nodeBuffer[i].do_stmt.next;
  body = g->nodeBuffer[i].do_stmt.body;
  cond = g->nodeBuffer[i].do_stmt.cond;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_WHILE_DO_LOOP;

  g->nodeBuffer[i].dc_asg_nd.file = -1;
  g->nodeBuffer[i].dc_asg_nd.line = line;

  /* store chain in other_node, since datrix nodes do not have <next> ptrs */
  g->nodeBuffer[i].common.other_node = next;

  /* arc cond to condition node */
  new_node = newNode(g);
  if (new_node == -1){
    fprintf(stderr,"error allocating node\n");
    exit(1);
  }
  g->nodeBuffer[new_node].common.other_node = -1;
  g->nodeBuffer[new_node].common.type = dC_ARC_COND;
  g->nodeBuffer[new_node].dc_arc.src = i;
  g->nodeBuffer[new_node].dc_arc.dest = cond;
  g->nodeBuffer[new_node].dc_arc.ordinal = 0;
  g->nodeBuffer[new_node].dc_arc.next = -1;
  g-> nodeBuffer[i].dc_do_while_loop.cond = new_node;

  g-> nodeBuffer[i].dc_do_while_loop.block = -1;
  prev_node = -1;
  while(body != -1){
    if (g->nodeBuffer[body].common.type == gSCOPE_STMT) {
      body = g->nodeBuffer[body].scope_stmt.next;
    } else {
      new_node = newNode(g);
      if (new_node == -1){
	fprintf(stderr,"error allocating node\n");
	exit(1);
      }
      g->nodeBuffer[new_node].common.other_node = -1;
      g->nodeBuffer[new_node].common.type = dC_ARC_SON;
      g->nodeBuffer[new_node].dc_arc.src = i;
      g->nodeBuffer[new_node].dc_arc.dest = body;
      g->nodeBuffer[new_node].dc_arc.ordinal = 0;
      g->nodeBuffer[new_node].dc_arc.next = -1;
      if (prev_node != -1){
	g->nodeBuffer[prev_node].dc_arc.next = new_node;
      }
      prev_node = new_node;
      if (g->nodeBuffer[i].dc_do_while_loop.block == -1){
	g->nodeBuffer[i].dc_do_while_loop.block = new_node;
      }
      tmp = body;
      body = nextNode(g,body);
      /* erase the chain*/
      setNext(g,tmp,-1);
    }
  }
}

node_ptr nextNode(gd_graph * g, node_ptr i){
  /* note - assumes that converted Datrix nodes
   * use other to point to next node */
  switch(g->nodeBuffer[i].common.type){
    case gNAMESPACE_DECL:
    case gFUNCTION_DECL:
    case gVAR_DECL:
    case gPARM_DECL:
    case gTYPE_DECL:
    case gLABEL_DECL:
    case gCONST_DECL:
    case gRESULT_DECL:
    case gFIELD_DECL:
    case gTEMPLATE_DECL:
    case gUSING_DECL:
      return g->nodeBuffer[i].decl.chan;
    case gFUNCTION_TYPE:
    case gINTEGER_TYPE:
    case gARRAY_TYPE:
    case gVOID_TYPE:
    case gREAL_TYPE:
    case gCOMPLEX_TYPE:
    case gVECTOR_TYPE:
    case gENUMERAL_TYPE:
    case gBOOLEAN_TYPE:
    case gCHAR_TYPE:
    case gPOINTER_TYPE:
    case gOFFSET_TYPE:
    case gREFERENCE_TYPE:
    case gMETHOD_TYPE:
    case gFILE_TYPE:
    case gSET_TYPE:
    case gRECORD_TYPE:
    case gUNION_TYPE:
    case gQUAL_UNION_TYPE:
    case gLANG_TYPE:
    case gTEMPLATE_TYPE_PARM:
    case gTEMPLATE_PARM_INDEX:
    case gTEMPLATE_TEMPLATE_PARM:
    case gBOUND_TEMPLATE_TEMPLATE_PARM:
    case gTYPENAME_TYPE:
    case gTYPEOF_TYPE:
    case gBLOCK:
      return -1;
    case gCOMPOUND_STMT:
      return g->nodeBuffer[i].compound_stmt.next;
    case gEXPR_STMT:
      return g->nodeBuffer[i].expr_stmt.next;
    case gDECL_STMT:
      return g->nodeBuffer[i].decl_stmt.next;
    case gSCOPE_STMT:
      return g->nodeBuffer[i].scope_stmt.next;
    case gRETURN_STMT:
      return g->nodeBuffer[i].return_stmt.next;
    case gSTMT_EXPR:
      return -1;
    case gBREAK_STMT:
      return g->nodeBuffer[i].break_stmt.next;
    case gCONTINUE_STMT:
      return g->nodeBuffer[i].continue_stmt.next;
    case gIF_STMT:
      return g->nodeBuffer[i].if_stmt.next;
    case gSWITCH_STMT:
      return g->nodeBuffer[i].switch_stmt.next;
    case gLABEL_STMT:
      return g->nodeBuffer[i].label_stmt.next;
    case gTRY_BLOCK:
      return g->nodeBuffer[i].try_block.next;
    case gHANDLER:
      return g->nodeBuffer[i].handler.next;
    case gSTART_CATCH_STMT:
      return g->nodeBuffer[i].start_catch_stmt.next;
    case gCLEANUP_STMT:
      return g->nodeBuffer[i].cleanup_stmt.next;
    case gCASE_LABEL:
      return g->nodeBuffer[i].case_label.next;
    case gGOTO_STMT:
      return g->nodeBuffer[i].goto_stmt.next;
    case gFOR_STMT:
      return g->nodeBuffer[i].for_stmt.next;
    case gWHILE_STMT:
      return g->nodeBuffer[i].while_stmt.next;
    case gDO_STMT:
      return g->nodeBuffer[i].do_stmt.next;
    case gASM_STMT:
      return g->nodeBuffer[i].asm_stmt.next;
    case gCOMPONENT_REF:
    case gBIT_FIELD_REF:
    case gINDIRECT_REF:
    case gBUFFER_REF:
    case gARRAY_REF:
    case gOFFSET_REF:
    case gSCOPE_REF:
    case gMEMBER_REF:
    case gCONSTRUCTOR:
    case gCOMPOUND_EXPR:
    case gMODIFY_EXPR:
    case gINIT_EXPR:
    case gTHROW_EXPR:
    case gTARGET_EXPR:
    case gCOND_EXPR:
    case gBIND_EXPR:
    case gCALL_EXPR:
    case gMETHOD_CALL_EXPR:
    case gWITH_CLEANUP_EXPR:
    case gCLEANUP_POINT_EXPR:
    case gPLACEHOLDER_EXPR:
    case gWITH_RECORD_EXPR:
    case gPLUS_EXPR:
    case gMINUS_EXPR:
    case gMULT_EXPR:
    case gTRUNC_DIV_EXPR:
    case gCEIL_DIV_EXPR:
    case gFLOOR_DIV_EXPR:
    case gROUND_DIV_EXPR:
    case gTRUNC_MOD_EXPR:
    case gCEIL_MOD_EXPR:
    case gFLOOR_MOD_EXPR:
    case gROUND_MOD_EXPR:
    case gRDIV_EXPR:
    case gEXACT_DIV_EXPR:
    case gFIX_TRUNC_EXPR:
    case gFIX_CEIL_EXPR:
    case gFIX_FLOOR_EXPR:
    case gFIX_ROUND_EXPR:
    case gFLOAT_EXPR:
    case gEXPON_EXPR:
    case gNEGATE_EXPR:
    case gMIN_EXPR:
    case gMAX_EXPR:
    case gABS_EXPR:
    case gFFS_EXPR:
    case gLSHIFT_EXPR:
    case gRSHIFT_EXPR:
    case gLROTATE_EXPR:
    case gRROTATE_EXPR:
    case gBIT_IOR_EXPR:
    case gBIT_XOR_EXPR:
    case gBIT_AND_EXPR:
    case gBIT_ANDTC_EXPR:
    case gTRUTH_ANDIF_EXPR:
    case gTRUTH_ORIF_EXPR:
    case gTRUTH_AND_EXPR:
    case gTRUTH_OR_EXPR:
    case gTRUTH_XOR_EXPR:
    case gTRUTH_NOT_EXPR:
    case gLT_EXPR:
    case gLE_EXPR:
    case gGT_EXPR:
    case gGE_EXPR:
    case gEQ_EXPR:
    case gNE_EXPR:
    case gUNORDERED_EXPR:
    case gORDERED_EXPR:
    case gUNLT_EXPR:
    case gUNLE_EXPR:
    case gUNGT_EXPR:
    case gUNGE_EXPR:
    case gUNEQ_EXPR:
    case gIN_EXPR:
    case gSET_LE_EXPR:
    case gCARD_EXPR:
    case gRANGE_EXPR:
    case gCONVERT_EXPR:
    case gNOP_EXPR:
    case gSAVE_EXPR:
    case gNON_LVALUE_EXPR:
    case gUNSAVE_EXPR:
    case gADDR_EXPR:
    case gREFERENCE_EXPR:
    case gRTL_EXPR:
    case gENTRY_VALUE_EXPR:
    case gCOMPLEX_EXPR:
    case gCONJ_EXPR:
    case gREALPART_EXPR:
    case gPREINCREMENT_EXPR:
    case gPREDECREMENT_EXPR:
    case gPOSTDECREMENT_EXPR:
    case gPOSTINCREMENT_EXPR:
    case gVA_ARG_EXPR:
    case gTRY_FINALLY_EXPR:
    case gTRY_CATCH_EXPR:
    case gPOPDHC_EXPR:
    case gPOPDCC_EXPR:
    case gGOTO_SUBROUTINE_EXPR:
    case gLABEL_EXPR:
    case gGOTO_EXPR:
    case gRETURN_EXPR:
    case gLOOP_EXPR:
    case gEXIT_EXPR:
    case gLABELED_BLOCK_EXPR:
    case gEXIT_BLOCK_EXPR:
    case gEXPR_WITH_FILE_LOCATION:
    case gSWITCH_EXPR:
    case gSIZEOF_EXPR:
    case gARROW_EXPR:
    case gNEW_EXPR:
    case gALIGNOF_EXPR:
    case gVEC_NEW_EXPR:
    case gDELETE_EXPR:
    case gVEC_DELETE_EXPR:
    case gTYPE_EXPR:
    case gAGGR_INIT_EXPR:
    case gEMPTY_CLASS_EXPR:
    case gTEMPLATE_ID_EXPR:
    case gLOOKUP_EXPR:
    case gFUNCTION_NAME:
      return -1;
    case gSUBOBJECT:
      return g->nodeBuffer[i].subobject.next;
    case gCTOR_STMT:
      return g->nodeBuffer[i].ctor_stmt.next;
    case gCTOR_INITIALIZER:
    case gRETURN_INIT:
    case gTAG_DEFN:
    case gIDENTITY_CONV:
    case gLVALUE_CONV:
    case gQUAL_CONV:
    case gSTD_CONV:
    case gPTR_CONV:
    case gPMEM_CONV:
    case gBASE_CONV:
    case gREF_BIND:
    case gUSER_CONV:
    case gRVALUE_CONV:
    case gOP_IDENTIFIER:
    case gIDENTIFIER_NODE:
    case gINTEGER_CST:
    case gREAL_CST:
    case gPTRMEM_CST:
    case gCOMPLEX_CST:
    case gSTRING_CST:
      return -1;
    case gTREE_LIST:
      return g->nodeBuffer[i].tree_list.chan;
    case gTREE_VEC:
    case gERROR_MARK:
    case gDEFAULT_ARG:
    case gCPLUS_BINDING:
      return -1;
    case gOVERLOAD:
      return g->nodeBuffer[i].overload.chan;
    case gWRAPPER:
    case NODE_DELTED:
    case NODE_NONE:
      return -1;
    /* datrix types */
    case dC_ASG_ND:
    case dC_IDENTIFIER:
    case dC_TYPE:
    case dC_BUILT_IN_TYPE:
    case dC_AGGR_TYPE:
    case dC_ENUM_TYPE:
    case dC_TEMPL_PARM_TYPE:
    case dC_GENERIC_TYPE:
    case dC_GENERATED_TYPE:
    case dC_ALIAS_TYPE:
    case dC_FORWARD_TYPE:
    case dC_ARRAY_TYPE:
    case dC_PTR_TYPE:
    case dC_REF_TYPE:
    case dC_FCT_TYPE:
    case dC_SCOPE:
    case dC_SCOPE_GLB:
    case dC_SCOPE_COMPIL:
    case dC_SCOPE_FILE:
    case dC_PACKAGE:
    case dC_NAME_SPACE:
    case dC_BLOCK:
    case dC_STATIC_BLOCK:
    case dC_TRY_BLOCK:
    case dC_CATCH_BLOCK:
    case dC_FINALLY_BLOCK:
    case dC_SYNCHRONIZED_BLOCK:
    case dC_OBJECT:
    case dC_FORMAL_FCT_PARAM:
    case dC_FORMAL_CATCH_PARAM:
    case dC_FUNCTION:
    case dC_LABEL:
    case dC_CASE_LABEL:
    case dC_DEFAULT_LABEL:
    case dC_ENUMERATOR:
    case dC_EXPRESSION:
    case dC_EMPTY_EXPR:
    case dC_LITERAL:
    case dC_SYNTAX_ERROR:
    case dC_NAME_REF:
    case dC_OPERATOR:
    case dC_UNARY_OPERATOR:
    case dC_BINARY_OPERATOR:
    case dC_QUESTION_OPERATOR:
    case dC_INIT_LIST:
    case dC_ACTUAL_FCT_PARAM_LIST:
    case dC_ACTUAL_TEMPL_PARAM_LIST:
    case dC_CREATION_EXPR:
    case dC_DESTROY_EXPR:
    case dC_CNTRL_STMT:
    case dC_SELECTION:
    case dC_IF:
    case dC_SWITCH:
    case dC_JUMP:
    case dC_GOTO:
    case dC_RETURN:
    case dC_BREAK:
    case dC_CONTINUE:
    case dC_LOOP:
    case dC_DO_WHILE_LOOP:
    case dC_WHILE_DO_LOOP:
    case dC_FOR_LOOP:
    case dC_SQL_STMT:
    case dC_COMMENT:
    case dC_JAVA_DOC_COMMENT:
    case dC_THROW_SPEC:
    case dC_ASM_BLOCK:
      return g->nodeBuffer[i].common.other_node;
    case dC_ARC:
    case dC_ARC_REFERENCE:
    case dC_INSTANCE:
    case dC_INHERITANCE:
    case dC_IMPLEMENTS:
    case dC_ARC_FRIEND:
    case dC_ARC_DEF_LOC:
    case dC_REFERS_TO:
    case dC_ARC_SYNTAX:
    case dC_ARC_SON:
    case dC_ARC_TRUE:
    case dC_ARC_FALSE:
    case dC_ARC_OPD:
    case dC_IMPORT:
    case dC_ARC_COND:
    case dC_ARC_FOR_INIT:
    case dC_ARC_FOR_INCR:
    case dC_ARC_INIT_VAL:
    case dC_ARC_BIT_DIM:
    case dC_ARC_ARRAY_DIM:
      return g->nodeBuffer[i].dc_arc.next;
    default: {
      printf("unknown node type: %d\n" ,g->nodeBuffer[i].common.type);
      exit(1);
    }
  }
}

void setNext(gd_graph * g, node_ptr i, node_ptr val){
  /* note - assumes that converted Datrix nodes
   * use other to point to next node */
  switch(g->nodeBuffer[i].common.type){
    case gNAMESPACE_DECL:
    case gFUNCTION_DECL:
    case gVAR_DECL:
    case gPARM_DECL:
    case gTYPE_DECL:
    case gLABEL_DECL:
    case gCONST_DECL:
    case gRESULT_DECL:
    case gFIELD_DECL:
    case gTEMPLATE_DECL:
    case gUSING_DECL:
      g->nodeBuffer[i].decl.chan = val;
      break;
    case gFUNCTION_TYPE:
    case gINTEGER_TYPE:
    case gARRAY_TYPE:
    case gVOID_TYPE:
    case gREAL_TYPE:
    case gCOMPLEX_TYPE:
    case gVECTOR_TYPE:
    case gENUMERAL_TYPE:
    case gBOOLEAN_TYPE:
    case gCHAR_TYPE:
    case gPOINTER_TYPE:
    case gOFFSET_TYPE:
    case gREFERENCE_TYPE:
    case gMETHOD_TYPE:
    case gFILE_TYPE:
    case gSET_TYPE:
    case gRECORD_TYPE:
    case gUNION_TYPE:
    case gQUAL_UNION_TYPE:
    case gLANG_TYPE:
    case gTEMPLATE_TYPE_PARM:
    case gTEMPLATE_PARM_INDEX:
    case gTEMPLATE_TEMPLATE_PARM:
    case gBOUND_TEMPLATE_TEMPLATE_PARM:
    case gTYPENAME_TYPE:
    case gTYPEOF_TYPE:
    case gBLOCK:
      break;
    case gCOMPOUND_STMT:
      g->nodeBuffer[i].compound_stmt.next = val;
      break;
    case gEXPR_STMT:
      g->nodeBuffer[i].expr_stmt.next = val;
      break;
    case gDECL_STMT:
      g->nodeBuffer[i].decl_stmt.next = val;
      break;
    case gSCOPE_STMT:
      g->nodeBuffer[i].scope_stmt.next = val;
      break;
    case gRETURN_STMT:
      g->nodeBuffer[i].return_stmt.next = val;
      break;
    case gSTMT_EXPR:
      break;
    case gBREAK_STMT:
      g->nodeBuffer[i].break_stmt.next = val;
      break;
    case gCONTINUE_STMT:
      g->nodeBuffer[i].continue_stmt.next = val;
      break;
    case gIF_STMT:
      g->nodeBuffer[i].if_stmt.next = val;
      break;
    case gSWITCH_STMT:
      g->nodeBuffer[i].switch_stmt.next = val;
      break;
    case gLABEL_STMT:
      g->nodeBuffer[i].label_stmt.next = val;
      break;
    case gTRY_BLOCK:
      g->nodeBuffer[i].try_block.next = val;
      break;
    case gHANDLER:
      g->nodeBuffer[i].handler.next = val;
      break;
    case gSTART_CATCH_STMT:
      g->nodeBuffer[i].start_catch_stmt.next = val;
      break;
    case gCLEANUP_STMT:
      g->nodeBuffer[i].cleanup_stmt.next = val;
      break;
    case gCASE_LABEL:
      g->nodeBuffer[i].case_label.next = val;
      break;
    case gGOTO_STMT:
      g->nodeBuffer[i].goto_stmt.next = val;
      break;
    case gFOR_STMT:
      g->nodeBuffer[i].for_stmt.next = val;
      break;
    case gWHILE_STMT:
      g->nodeBuffer[i].while_stmt.next = val;
      break;
    case gDO_STMT:
      g->nodeBuffer[i].do_stmt.next = val;
      break;
    case gASM_STMT:
      g->nodeBuffer[i].asm_stmt.next = val;
      break;
    case gCOMPONENT_REF:
    case gBIT_FIELD_REF:
    case gINDIRECT_REF:
    case gBUFFER_REF:
    case gARRAY_REF:
    case gOFFSET_REF:
    case gSCOPE_REF:
    case gMEMBER_REF:
    case gCONSTRUCTOR:
    case gCOMPOUND_EXPR:
    case gMODIFY_EXPR:
    case gINIT_EXPR:
    case gTHROW_EXPR:
    case gTARGET_EXPR:
    case gCOND_EXPR:
    case gBIND_EXPR:
    case gCALL_EXPR:
    case gMETHOD_CALL_EXPR:
    case gWITH_CLEANUP_EXPR:
    case gCLEANUP_POINT_EXPR:
    case gPLACEHOLDER_EXPR:
    case gWITH_RECORD_EXPR:
    case gPLUS_EXPR:
    case gMINUS_EXPR:
    case gMULT_EXPR:
    case gTRUNC_DIV_EXPR:
    case gCEIL_DIV_EXPR:
    case gFLOOR_DIV_EXPR:
    case gROUND_DIV_EXPR:
    case gTRUNC_MOD_EXPR:
    case gCEIL_MOD_EXPR:
    case gFLOOR_MOD_EXPR:
    case gROUND_MOD_EXPR:
    case gRDIV_EXPR:
    case gEXACT_DIV_EXPR:
    case gFIX_TRUNC_EXPR:
    case gFIX_CEIL_EXPR:
    case gFIX_FLOOR_EXPR:
    case gFIX_ROUND_EXPR:
    case gFLOAT_EXPR:
    case gEXPON_EXPR:
    case gNEGATE_EXPR:
    case gMIN_EXPR:
    case gMAX_EXPR:
    case gABS_EXPR:
    case gFFS_EXPR:
    case gLSHIFT_EXPR:
    case gRSHIFT_EXPR:
    case gLROTATE_EXPR:
    case gRROTATE_EXPR:
    case gBIT_IOR_EXPR:
    case gBIT_XOR_EXPR:
    case gBIT_AND_EXPR:
    case gBIT_ANDTC_EXPR:
    case gTRUTH_ANDIF_EXPR:
    case gTRUTH_ORIF_EXPR:
    case gTRUTH_AND_EXPR:
    case gTRUTH_OR_EXPR:
    case gTRUTH_XOR_EXPR:
    case gTRUTH_NOT_EXPR:
    case gLT_EXPR:
    case gLE_EXPR:
    case gGT_EXPR:
    case gGE_EXPR:
    case gEQ_EXPR:
    case gNE_EXPR:
    case gUNORDERED_EXPR:
    case gORDERED_EXPR:
    case gUNLT_EXPR:
    case gUNLE_EXPR:
    case gUNGT_EXPR:
    case gUNGE_EXPR:
    case gUNEQ_EXPR:
    case gIN_EXPR:
    case gSET_LE_EXPR:
    case gCARD_EXPR:
    case gRANGE_EXPR:
    case gCONVERT_EXPR:
    case gNOP_EXPR:
    case gSAVE_EXPR:
    case gNON_LVALUE_EXPR:
    case gUNSAVE_EXPR:
    case gADDR_EXPR:
    case gREFERENCE_EXPR:
    case gRTL_EXPR:
    case gENTRY_VALUE_EXPR:
    case gCOMPLEX_EXPR:
    case gCONJ_EXPR:
    case gREALPART_EXPR:
    case gPREINCREMENT_EXPR:
    case gPREDECREMENT_EXPR:
    case gPOSTDECREMENT_EXPR:
    case gPOSTINCREMENT_EXPR:
    case gVA_ARG_EXPR:
    case gTRY_FINALLY_EXPR:
    case gTRY_CATCH_EXPR:
    case gPOPDHC_EXPR:
    case gPOPDCC_EXPR:
    case gGOTO_SUBROUTINE_EXPR:
    case gLABEL_EXPR:
    case gGOTO_EXPR:
    case gRETURN_EXPR:
    case gLOOP_EXPR:
    case gEXIT_EXPR:
    case gLABELED_BLOCK_EXPR:
    case gEXIT_BLOCK_EXPR:
    case gEXPR_WITH_FILE_LOCATION:
    case gSWITCH_EXPR:
    case gSIZEOF_EXPR:
    case gARROW_EXPR:
    case gNEW_EXPR:
    case gALIGNOF_EXPR:
    case gVEC_NEW_EXPR:
    case gDELETE_EXPR:
    case gVEC_DELETE_EXPR:
    case gTYPE_EXPR:
    case gAGGR_INIT_EXPR:
    case gEMPTY_CLASS_EXPR:
    case gTEMPLATE_ID_EXPR:
    case gLOOKUP_EXPR:
    case gFUNCTION_NAME:
      break;
    case gSUBOBJECT:
      g->nodeBuffer[i].subobject.next = val;
      break;
    case gCTOR_STMT:
      g->nodeBuffer[i].ctor_stmt.next = val;
      break;
    case gCTOR_INITIALIZER:
    case gRETURN_INIT:
    case gTAG_DEFN:
    case gIDENTITY_CONV:
    case gLVALUE_CONV:
    case gQUAL_CONV:
    case gSTD_CONV:
    case gPTR_CONV:
    case gPMEM_CONV:
    case gBASE_CONV:
    case gREF_BIND:
    case gUSER_CONV:
    case gRVALUE_CONV:
    case gOP_IDENTIFIER:
    case gIDENTIFIER_NODE:
    case gINTEGER_CST:
    case gREAL_CST:
    case gPTRMEM_CST:
    case gCOMPLEX_CST:
    case gSTRING_CST:
      break;
    case gTREE_LIST:
      g->nodeBuffer[i].tree_list.chan = val;
      break;
    case gTREE_VEC:
    case gERROR_MARK:
    case gDEFAULT_ARG:
    case gCPLUS_BINDING:
      break;
    case gOVERLOAD:
      g->nodeBuffer[i].overload.chan = val;
      break;
    case gWRAPPER:
    case NODE_DELTED:
    case NODE_NONE:
      break;
    /* datrix types */
    case dC_ASG_ND:
    case dC_IDENTIFIER:
    case dC_TYPE:
    case dC_BUILT_IN_TYPE:
    case dC_AGGR_TYPE:
    case dC_ENUM_TYPE:
    case dC_TEMPL_PARM_TYPE:
    case dC_GENERIC_TYPE:
    case dC_GENERATED_TYPE:
    case dC_ALIAS_TYPE:
    case dC_FORWARD_TYPE:
    case dC_ARRAY_TYPE:
    case dC_PTR_TYPE:
    case dC_REF_TYPE:
    case dC_FCT_TYPE:
    case dC_SCOPE:
    case dC_SCOPE_GLB:
    case dC_SCOPE_COMPIL:
    case dC_SCOPE_FILE:
    case dC_PACKAGE:
    case dC_NAME_SPACE:
    case dC_BLOCK:
    case dC_STATIC_BLOCK:
    case dC_TRY_BLOCK:
    case dC_CATCH_BLOCK:
    case dC_FINALLY_BLOCK:
    case dC_SYNCHRONIZED_BLOCK:
    case dC_OBJECT:
    case dC_FORMAL_FCT_PARAM:
    case dC_FORMAL_CATCH_PARAM:
    case dC_FUNCTION:
    case dC_LABEL:
    case dC_CASE_LABEL:
    case dC_DEFAULT_LABEL:
    case dC_ENUMERATOR:
    case dC_EXPRESSION:
    case dC_EMPTY_EXPR:
    case dC_LITERAL:
    case dC_SYNTAX_ERROR:
    case dC_NAME_REF:
    case dC_OPERATOR:
    case dC_UNARY_OPERATOR:
    case dC_BINARY_OPERATOR:
    case dC_QUESTION_OPERATOR:
    case dC_INIT_LIST:
    case dC_ACTUAL_FCT_PARAM_LIST:
    case dC_ACTUAL_TEMPL_PARAM_LIST:
    case dC_CREATION_EXPR:
    case dC_DESTROY_EXPR:
    case dC_CNTRL_STMT:
    case dC_SELECTION:
    case dC_IF:
    case dC_SWITCH:
    case dC_JUMP:
    case dC_GOTO:
    case dC_RETURN:
    case dC_BREAK:
    case dC_CONTINUE:
    case dC_LOOP:
    case dC_DO_WHILE_LOOP:
    case dC_WHILE_DO_LOOP:
    case dC_FOR_LOOP:
    case dC_SQL_STMT:
    case dC_COMMENT:
    case dC_JAVA_DOC_COMMENT:
    case dC_THROW_SPEC:
    case dC_ASM_BLOCK:
      g->nodeBuffer[i].common.other_node = val;
      break;
    case dC_ARC:
    case dC_ARC_REFERENCE:
    case dC_INSTANCE:
    case dC_INHERITANCE:
    case dC_IMPLEMENTS:
    case dC_ARC_FRIEND:
    case dC_ARC_DEF_LOC:
    case dC_REFERS_TO:
    case dC_ARC_SYNTAX:
    case dC_ARC_SON:
    case dC_ARC_TRUE:
    case dC_ARC_FALSE:
    case dC_ARC_OPD:
    case dC_IMPORT:
    case dC_ARC_COND:
    case dC_ARC_FOR_INIT:
    case dC_ARC_FOR_INCR:
    case dC_ARC_INIT_VAL:
    case dC_ARC_BIT_DIM:
    case dC_ARC_ARRAY_DIM:
      g->nodeBuffer[i].dc_arc.next = val;
      break;
    default: {
      printf("unknown node type: %d\n" ,g->nodeBuffer[i].common.type);
      exit(1);
    }
  }
}
