/*+
 * Name:	namespace.c
 *
 * Purpose:	Transforms gcc namespace_decl to datrix namespace.
 *              -- transfomrs decl ptr in gcc namespace to arcSon
 *              -- moves name from identifier node dircetly to namespace node
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 7, 2001
 *
 * Revision History
 *
 * 1.1	 March 12, 2001 TRD
 *       - delete chan pointer (set to -1) since Arc son now contains
 *         the ordering relationship.
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  str_ptr internal;


  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  internal = gd_lookup_string(&g,"<internal>");

  for(i = 0; i < g.curNodePos;i++){
    if (g.nodeBuffer[i].common.type == gNAMESPACE_DECL){
      /* fields to save */
      node_ptr name;
      str_ptr srcf;
      int srcl;
      node_ptr contents; /* dcls or alias depending on fields */
      int isAlis;
      node_ptr tmp;

      name = g.nodeBuffer[i].decl.name;
      srcf = g.nodeBuffer[i].decl.srcf;
      srcl = g.nodeBuffer[i].decl.srcl;
      isAlis=0;
      contents = -1;
      if (g.nodeBuffer[i].namespace_decl.dcls != -1){
	contents = g.nodeBuffer[i].namespace_decl.dcls;
      }
      if (g.nodeBuffer[i].namespace_decl.alis != -1){
	contents = g.nodeBuffer[i].namespace_decl.alis;
	isAlis = 1;
      }

      /* switch node type */
      g.nodeBuffer[i].common.type = dC_NAME_SPACE;

      /* unused node_ptr subfields */
      g.nodeBuffer[i].dc_identifier.instance = -1;
      g.nodeBuffer[i].dc_identifier.defLoc = -1;
      g.nodeBuffer[i].dc_identifier.arcFriend = -1;

      /* determin datrix visibility from common flags */
      /* gd_common doesn't change, no need to have saved flags */

      g.nodeBuffer[i].dc_identifier.visb = visb_unk;
      if (g.nodeBuffer[i].common.private_flag){
	g.nodeBuffer[i].dc_identifier.visb = visb_pri;
      }
      if (g.nodeBuffer[i].common.public_flag){
	g.nodeBuffer[i].dc_identifier.visb = visb_pub;
      }
      if (g.nodeBuffer[i].common.protected_flag){
	g.nodeBuffer[i].dc_identifier.visb = visb_pro;
      }

      /* if there is an identifier node, then move the str_ptr
       * from the identifier node to the datrix identifier part of
       * the new node */

      if (name != -1) {
	g.nodeBuffer[i].dc_identifier.name = g.nodeBuffer[name].identifier_node.strg;
	/* check to see if identifier node is used */
	if (UsedBy(g,name,0) == -1){
	  /* id node is not used */
	  /* delete it */
	  g.nodeBuffer[name].common.type = NODE_DELTED;
	} else {
	  printf("node %d used by %d\n",name,UsedBy(g,name,0));
	}
      } else {
	/* set str_ptr to -1 for no string */
	g.nodeBuffer[i].dc_identifier.name = -1;
      }
      /* source file and line */
      g.nodeBuffer[i].dc_asg_nd.line = srcl;
      g.nodeBuffer[i].dc_asg_nd.file = srcf;
      /* gcc has the notion of an alias namespace, TODO */
      if (isAlis){
	/* one child, which is another namespace */
	/* datrix manual does not discuss this, nor is it in thier examples */
	/* model as a typedef , that is make an instance of edge */
	node_ptr new_node = newNode(&g);
	g.nodeBuffer[new_node].common.type = dC_INSTANCE;
	g.nodeBuffer[new_node].common.other_node = -1;
	g.nodeBuffer[new_node].dc_arc.ordinal = 0;
	g.nodeBuffer[new_node].dc_arc.next = -1;
	g.nodeBuffer[new_node].dc_arc.src = i;
	g.nodeBuffer[new_node].dc_arc.dest = contents;
	g.nodeBuffer[i].dc_scope.contents = new_node;
      } else {
	/* allocate a set of arc nodes implemnting the arcSon relation */
	/* travers the chan of decls given by the dcls pointer */
	g.nodeBuffer[i].dc_scope.contents = -1;
	if (contents != -1){
	  node_ptr new_node = newNode(&g);
	  g.nodeBuffer[new_node].common.type = dC_ARC_SON;
	  g.nodeBuffer[new_node].dc_arc.src = i;
	  if (internal != -1) {
	    if (g.nodeBuffer[contents].decl.srcf == internal &&
		g.nodeBuffer[g.nodeBuffer[contents].decl.type].common.type == dC_BUILT_IN_TYPE){
	      g.nodeBuffer[new_node].dc_arc.dest = g.nodeBuffer[contents].decl.type;
	    } else {
	      g.nodeBuffer[new_node].dc_arc.dest = contents;
	    }
	  } else {
	    /* no internal nodes, no type_decl compaction here */
	    g.nodeBuffer[new_node].dc_arc.dest = contents;
	  }
	  g.nodeBuffer[new_node].dc_arc.ordinal = 0;
	  g.nodeBuffer[i].dc_scope.contents = new_node;
	  tmp = contents;
	  contents = g.nodeBuffer[contents].decl.chan;
	  g.nodeBuffer[tmp].decl.chan = -1; /*1.1 TRD*/
	  while (contents != -1){
	    node_ptr prev_node = new_node;
	    new_node = newNode(&g);
	    g.nodeBuffer[new_node].common.type = dC_ARC_SON;
	    g.nodeBuffer[new_node].dc_arc.src = i;
	    if (internal != -1){
	      if (g.nodeBuffer[contents].decl.srcf == internal &&
		  g.nodeBuffer[g.nodeBuffer[contents].decl.type].common.type == dC_BUILT_IN_TYPE){
		g.nodeBuffer[new_node].dc_arc.dest = g.nodeBuffer[contents].decl.type;
	      } else {
		g.nodeBuffer[new_node].dc_arc.dest = contents;
	      }
	    } else {
	      g.nodeBuffer[new_node].dc_arc.dest = contents;
	    }
	    g.nodeBuffer[new_node].dc_arc.ordinal = 0;
	    /* link arcsons in a given relation together for quick traversal */
	    g.nodeBuffer[prev_node].dc_arc.next = new_node;
	    tmp = contents;
	    contents = g.nodeBuffer[contents].decl.chan;
	    g.nodeBuffer[tmp].decl.chan = -1; /*1.1 TRD*/
	  }
	  g.nodeBuffer[new_node].dc_arc.next = -1;
	}
      }
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}
