/*+
 * Name:	zap_builtin.c
 *
 * Purpose:	Removes built in types if not refernced by any nodes
 *
 * Author:	Thomas R. Dean
 *
 * Date:	May 4, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  node_ptr *rover;
  int changed;

  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  /* start at front node */
  for(i = 0; i < g.curNodePos; i++){
    g.nodeBuffer[i].common.mark_flag1 = 0;
  }

  if (g.nodeBuffer[g.firstNode].common.type != dC_NAME_SPACE){
    fprintf(stderr,"first node is not a namespace node!!\n");
    exit(1);
  }

  rover = &(g.nodeBuffer[g.firstNode].dc_scope.contents);
  if (*rover != -1){
    if (g.nodeBuffer[*rover].common.type != dC_ARC_SON){
      fprintf(stderr,"contents of namespace not arcs!!\n");
      exit(1);
    }
  }

  while(*rover != -1) {
    node_ptr dest = g.nodeBuffer[*rover].dc_arc.dest;
    changed = 0;
    if (g.nodeBuffer[dest].common.type == dC_BUILT_IN_TYPE){

      /* the son is the built in type void */
      node_ptr uses;

      /* check to see if used by any other node than the current
       * ArcSon edge */
      g.nodeBuffer[*rover].common.mark_flag1 = 1;
      uses = UsedBy(g,dest,1);
      if (uses == -1){
	/* not used by any other node */
	/* delete the arc node, which will delete the
	 * void node */
	*rover = g.nodeBuffer[*rover].dc_arc.next;
	changed = 1;
      }
      g.nodeBuffer[*rover].common.mark_flag1 = 0;
    }

    /* go to the next node */
    if (!changed){
      rover = &(g.nodeBuffer[*rover].dc_arc.next);
    }
  }

  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}

